// --------------------------------------------------
// Ebenen-Deklaration
// --------------------------------------------------

SYN     [mainlevel=yes] # C1, C2, C3, C4, MOD;
OBL1    [mirror=C1] # "";
OBL2    [mirror=C2] # "";
OBL3    [mirror=C3] # "";
OBL4    [mirror=C4] # "";
OBL1    # "";
OBL2    # "";
OBL3    # "";
OBL4    # "";
VORFELD # "";
AGENT   # "";
EXP     # "";
PATIENT # "";
QUALITY # "";
THEME   # "";
SUB	# "";

/* --------------------------------------------------

ganz schwache Prferenzen

-------------------------------------------------- */
{X:SYN} : no_acc : case : 0.999 :
  exists( X@case )
  -> X@case!=acc
;
{X:SYN} : no_dat : case : 0.998 :
  exists( X@case )
  -> X@case!=dat
;
{X:SYN} : no_gen : case : 0.997 :
  exists( X@case )
  -> X@case!=gen
;

{X:SYN} : no_fem : gender : 0.9999 :
  exists( X@gender )
  -> X@gender!=fem
;
{X:SYN} : no_neut : gender : 0.9998 :
  exists( X@gender )
  -> X@gender!=neut
;

{X:SYN} : no_pl : number : 0.999 :
  exists( X@number )
  -> X@number!=pl
;

// die Fgung `in DATUM' o.. ist oft mehrdeutig, wenn sie nicht
// irgendwo als Komplement unterkommen kann, und aufgrund des
// unterspezifizierten DATUM schlgt auch die Kasusprferenz nicht zu.
// Daher prferieren wir hier auch noch den loctype.
{X:SYN} : no_lative : loctype : 0.999 :
  exists( X@loctype )
  -> X@loctype!=lative
;

/* --------------------------------------------------

Wurzelanbindung
===============

-------------------------------------------------- */

// Wurzelanbindungen sind MOD.
{X|SYN} : root_is_mod : 0.0 :
  X.label=MOD
;
// Wurzeln sind nur finite Verben
// oder Konjunktionen 
{X|SYN} : root_only_vfin : 0.01 :
  subsumes(Kategorien, finit, X@cat) | X@cat=KON
;
// aber bitte auch nicht mehr als eine solche Wurzel 
{X|SYN, Y|SYN} : only_one_root : 0.1 :
  (subsumes(Kategorien, finit, X@cat) | X@cat=KON)
& (subsumes(Kategorien, finit, Y@cat) | Y@cat=KON)
  ->
  false
;

/* --------------------------------------------------

Projektivitt
=============

Projektivitt wird grundstzlich erwnscht. Allerdings 
sind wohl an einigen Stellen Ausnahmen zu kodieren.

-------------------------------------------------- */

// Eine Konstituente wird nicht zwischen ihren Endpunkten angebunden.
//
// Ausnahmen werden fr komplexe Verbgruppen gemacht:
//
// X:       sollen(wir)<--------------C2--------------tun
// Y: was-------------------C3----------------------->tun
//
{X!SYN/Y!SYN} : proj1 : 0.0 :
  (spec(X^id) & subsumes(Kategorien, Hilfsverb, X^cat))
  |
  ( subsumes(Kategorien, KOUS, X@cat) &  Y.label = C1)
  |
  ( X^to <= min(Y@from, X@from) 
   |
   X^from >= max(Y@to, X@to)
  )
;

// Zwei Kanten berschneiden sich nicht
//
// Ausnahmen werden fr komplexe Verben gemacht:
//
// a) Im Objekt-Relativsatz kann das Subjekt
//    jenseits des Partizips stehen:
//
// `der Termin, den wir abgesprochen haben'
//
// X: den--------C2------>abgesprochen
// Y:       wir-------------C1----------->haben
//
// b) Im Hauptsatz kann Projektivitt immer dann verletzt sein,
//    wenn mehrere Bestandteile des Verbs eine Klammerstellung 
//    einnehmen:
//
// `wir sollten das frher machen mit dem Termin'
// 
// X: (wir)sollten<--------C2-------machen
// Y:                     das<------------MOD---------mit(dem Termin)
//
//
// `ich gebe den Plan auf, Profi zu werden'
//
// X: (ich)gebe<--------C4---------auf   
// Y:                (den)Plan<-----C1-----(Profi)zu_werden  
//
// Im allgemeinen: wenn X zwischen zwei Bestandteilen desselben Verbs verluft,
// dann darf X Projektivitt verletzen.

{X!SYN || Y!SYN} : proj2 : 0.0 :
 min(X@from, X^from) < min(Y@from, Y^from)
 & (spec(X^id) | spec(Y^id))
  -> (
      ( max(X@from, X^from) > max(Y@from, Y^from)) 
     | 
      ( max(X@from, X^from) < min(Y@from, Y^from))
     | // a)
      ( subsumes(Kategorien, Relativpronomen, X@cat)
        & Y.label=C1 )
     | // b)
      ( spec(X^id)
        & subsumes(Kategorien, Verbbestandteil, X^cat)
        & subsumes(Kategorien, Verbbestandteil, X@cat)
	& X.label != MOD)
     )
;


// Damit die Ausnahmen nicht unversehens zuschlagen,
// hier noch zwei zahnlose Varianten, die nur dazu dienen,
// auf Ausnahmesituationen hinzuweisen.
{X!SYN/Y!SYN} : nonprojective1 : 1.0 :
  ( X^to <= min(Y@from, X@from) 
   |
   X^from >= max(Y@to, X@to)
  )
;
{X!SYN || Y!SYN} : nonprojective2 : 1.0 :
 (
  min(X@from, X^from) < min(Y@from, Y^from)       
  & (spec(X^id) | spec(Y^id))
  -> (
      ( max(X@from, X^from) > max(Y@from, Y^from)) 
     | 
      ( max(X@from, X^from) < min(Y@from, Y^from)) 
    )
  )
;

/* --------------------------------------------------

Komplemente
===========

SYN::C1 bis SYN::C4 und OBL1 bis OBL4

-------------------------------------------------- */

// Komplemente sind stark eingeschrnkt bindungsfhig,
// aber auch sie sollten leicht fr Sperrung bestraft werden.
{X!SYN} : C_distance : [ 100 / [ 100 + abs( distance(X@id, X^id) ) ] ] :
  X.label!=MOD
  ->
 abs( distance( X@id, X^id ) ) <= 1
;


m4_define(M_c_init,`m4_dnl
// Komplemente hngen nur an Worten, die ein Komplement lizensieren.
{X:SYN} : c$1_init : 0.0 :
  spec(X^id) 
  & X.label=C$1 
  -> 
  X! 
  & exists(X^args:$1:obl)
;
{X!OBL$1} : obl$1_possible : redundant : 0.0 :
  exists(X@args:$1:obl)
;

// Wenn ein zu realisierendes Komplement fehlt, wird das bestraft.
// Wie hart die Strafe ist, entscheidet das Lexem in Eigenverantwortung.
{X|OBL$1} : obl$1_missing_penalty : [ X@args:$1:obl ] :
  exists(X@args:$1:obl)
  ->
  X@args:$1:realized = no
;

// Wenn ein nicht zu realisierendes Komplement existiert,
// wird das ebenfalls bestraft.
{X!OBL$1} : obl$1_exist_penalty : [ X@args:$1:obl ] :
  exists(X@args:$1:obl)
  ->
  X@args:$1:realized = yes
;

')


m4_define(M_c_pos, `m4_dnl

// Die Stellung von Komplementen wird gewhnlich ber die Kategorie
// abgehandelt (vgl. etwa pp_order), aber bei einigen Worten ist sie
// lexikalisiert: z.B. `ein' in `ein bichen'.


// rightpenalty bestraft nicht etwa korrekte Komplemente,
// sondern solche, die rechts stehen.
{X\SYN} : c$1_right : [ X^args:$1:rightpenalty ] :
  spec(X^id) 
  & X.label=C$1
  -> ~exists(X^args:$1:rightpenalty)
;
{X/OBL$1} : obl$1_right : [ X@args:$1:rightpenalty ] :
 ~exists(X@args:$1:rightpenalty)
;

// leftpenalty bestraft nicht vergessene Komplemente,
// sondern solche, die links stehen.
{X/SYN} : c$1_left : [ X^args:$1:leftpenalty ] :
  spec(X^id) & X.label=C$1
  -> ~exists(X^args:$1:leftpenalty)
;
{X\OBL$1} : obl$1_left : [ X@args:$1:leftpenalty ] :
 ~exists(X@args:$1:leftpenalty)
;


// Komplemente sind eindeutig.
// Das wird zwar implizit durch die Abbildung SYN--OBL gefordert,
// aber wir wiederholen die Forderung hier fr Nur-Syntax-Analysen.
{X!SYN/\Y!SYN} : c$1_uniq : 0.0 :
  X.label=C$1 -> Y.label!=C$1
;

')

m4_define(M_c_agree,`m4_dnl
// wenn args:$1:$2 den Wert X verlangt,
// dann mu das Komplement dieses Feature zumindest besitzen.
{X:SYN} : c$1_exists_$2 : 0.0 :
  spec(X^id)
  & X.label=C$1 
  & exists(X^args:$1:$2)
  -> 
  exists(X@$2)
;
// Wiederholung auf OBL-Ebene
{X!OBL$1} : obl$1_exists_$2 : redundant : 0.0 :
  spec(X^id) &
  exists(X@args:$1:$2)
  ->
  exists(X^$2)
;
// wenn args:N:FEATURE den Wert X verlangt,
// dann soll das Komplement denselben Wert aufweisen.
{X:SYN} : c$1_agree_$2 : $3 :
  spec(X^id)
  & X.label=C$1 
  & exists(X@$2)
  & exists(X^args:$1:$2) 
  -> 
  X^args:$1:$2=X@$2 
;
// Wiederholung auf OBL-Ebene
{X!OBL$1} : obl$1_agree_$2 : redundant : $3 :
  spec(X^id)
  & exists(X@args:$1:$2) 
  & exists(X^$2)
  -> 
  X@args:$1:$2=X^$2
;')

m4_define(M_c_subsume,`m4_dnl
// wie oben, aber mit subsumes() anstatt =
{X:SYN} : c$1_exists_$2 : 0.0 :
  spec(X^id)
  & X.label=C$1 
  & exists(X^args:$1:$2) 
  ->
  exists(X@$2)
;
{X!OBL$1} : obl$1_exists_$2 : redundant : 0.0 :
  spec(X^id)
  & exists(X@args:$1:$2) 
  ->
  exists(X^$2)
;
{X:SYN} : c$1_subsume_$2 : $3 :
  spec(X^id)
  & X.label=C$1 
  & exists(X^args:$1:$2) 
  & exists(X@$2)
  -> 
    // `Kategorien' ist die Defaulthierarchie.
    subsumes(m4_ifelse($4,,Kategorien,$4), X^args:$1:$2, X@$2)
;
{X!OBL$1} : obl$1_subsume_$2 : redundant : $3 :
  spec(X^id)
  & exists(X@args:$1:$2) 
  & exists(X^$2)
  -> 
  subsumes(m4_ifelse($4,,Kategorien,$4),X@args:$1:$2, X^$2)
;')


m4_define(M_c_obl_coupling, `m4_dnl
// syntaktisches C$1 mu von OBL$1 gespiegelt werden
{X!SYN \ Y:OBL$1} : c$1_obl$1_coupling1 : 0.0 :
  X.label=C$1
  ->
  X/Y
;
// OBL$1-Anbindung mu syntaktisch gespiegelt werden
// und dort C$1 heien
{X:SYN / Y!OBL$1} : c$1_obl$1_coupling2 : 0.0 :
  X.label=C$1
  & X\Y
;')


m4_define(M_c_compatible,`m4_dnl
// wie oben, aber mit compatible() anstatt =
{X:SYN} : c$1_exists_$2 : 0.0 :
  spec(X^id)
  & X.label=C$1 
  & exists(X^args:$1:$2) 
  ->
  exists(X@$2)
;
{X!OBL$1} : obl$1_exists_$2 : redundant : 0.0 :
  spec(X^id)
  & exists(X@args:$1:$2) 
  ->
  exists(X^$2)
;
{X:SYN} : c$1_subsume_$2 : $3 :
  spec(X^id)
  & X.label=C$1 
  & exists(X^args:$1:$2) 
  & exists(X@$2)
  -> 
    // `Kategorien' ist die Defaulthierarchie.
    compatible(m4_ifelse($4,,Kategorien,$4), X^args:$1:$2, X@$2)
;
{X!OBL$1} : obl$1_subsume_$2 : redundant : $3 :
  spec(X^id)
  & exists(X@args:$1:$2) 
  & exists(X^$2)
  -> 
  compatible(m4_ifelse($4,,Kategorien,$4),X@args:$1:$2, X^$2)
;')


m4_define(M_c_obl_coupling,`m4_dnl
// syntaktisches C$1 mu von OBL$1 gespiegelt werden
{X!SYN \ Y:OBL$1} : c$1_obl$1_coupling1 : 0.0 :
  X.label=C$1
  ->
  X/Y
;
// OBL$1-Anbindung mu syntaktisch gespiegelt werden
// und dort C$1 heien
{X:SYN / Y!OBL$1} : c$1_obl$1_coupling2 : 0.0 :
  X.label=C$1
  & X\Y
;')


m4_define(M_c_common,`m4_dnl
M_c_init($1)
M_c_obl_coupling($1)
M_c_pos($1)
M_c_subsume($1,cat,0.05$1)
M_c_agree($1,person,0.1$1)
M_c_agree($1,number,0.12$1)
M_c_compatible($1,case,0.15$1, Kasus)
M_c_compatible($1,gender,0.13$1, Gender)
M_c_agree($1,word,0.0)
M_c_agree($1,baseform,0.0)
M_c_agree($1,loctype,0.0)
M_c_agree($1,flexion,0.0)
')

M_c_common(1)
M_c_common(2)
M_c_common(3)
M_c_common(4)

// Das erste Komplement steht gewhnlich vor dem zweiten etc.
// (disambiguiert `ist das der 8. IV.?')
// 
// Ausnahmen werden gemacht fr `wo bist du?' 
// und fr `ich wei, was es ist'.
{X!SYN/\Y!SYN} : c_c_order : 0.9 :
 (
  (X.label=C1 & Y.label=C2)
  |
  (X.label=C1 & Y.label=C3)
  |
  (X.label=C1 & Y.label=C4)
  |
  (X.label=C2 & Y.label=C3)
  |
  (X.label=C2 & Y.label=C4)
  |
  (X.label=C3 & Y.label=C4)
 )
 ->
 distance(X@id, Y@id) > 0
 |
 subsumes(Kategorien, Interrogativpronomen, Y@cat) 
 |
 subsumes(Kategorien, unterordnend, Y@cat)
;

/* --------------------------------------------------

Bedingungen an PPs als Komplemente
==================================

Diese sind komplizierter als andere Argumente, weil sowohl 
der mittlere als auch der untere Knoten Features 
besitzen, die vom oberen Knoten lizensiert werden:

    o              
MOD |                   
    |
    o----
     \   \ 
    C1\   \C2
       \   \
        o   o
             \
              \C1
(SYN)          \
                \
                 o

   geht es bei Ihnen

            o
           /
(EXP)     /
         /
        /
       /
      /
     o

In diesem Fall stellt `gehen' etwa die Bedingungen:

    2:
    [ obl: no,
      case: bei,
      cat: APPR,
      prepsort: human,
      role: EXP
    ]

Case und cat werden an der C2-Kante abgefragt.
Aber da das Argument des Argumentes human 
sein soll, lt sich nur binr fordern:

{X:SYN/Y:SYN} : c2_subsume_prepsort : 0.1 :
  spec(X^id)
  & X.label = C2
  & Y.label = C1  
  & exists(X^args:2:prepsort) 
  & exists(Y@sort)
  ->
  subsumes(Sorten, X^args:2:prepsort, Y@sort);
-------------------------------------------------- */

m4_define(Sortencheck, `
// wenn X^ eine Sorte verlangt, 
// mu X@ zumindest irgendeine Sorte haben
{X:SYN} : c$1_exists_sort : 0.0 :
  spec(X^id)
  & X.label=C$1 
  & exists(X^args:$1:sort) 
  ->
  exists(X@sort)
;

// Wiederholung auf der OBL-Ebene
{X!OBL$1} : obl$1_exists_sort : redundant : 0.0 :
  spec(X^id)
  & exists(X@args:$1:sort) 
  ->
  exists(X^sort)
;

// wenn X^ eine Sorte verlangt, 
// soll X dieselbe Sorte haben
// (auer es ist eine PP --> das wird weiter unten behandelt)
{X!SYN} : c$1_subsume_sort : 0.3 :
  spec(X^id)
  & X.label=C$1 
  & exists(X^args:$1:sort) 
  & ~subsumes(Kategorien, Adposition, X@cat) 
  & exists(X@sort)
  -> 
  subsumes(Sorten, X^args:$1:sort, X@sort)
;

// Wiederholung auf der OBL-Ebene
{X!OBL$1} : obl$1_subsume_sort : redundant : 0.3 :
  spec(X^id)
  & exists(X@args:$1:sort) 
  & ~subsumes(Kategorien, Adposition, X^cat) 
  & exists(X^sort)
  -> 
  subsumes(Sorten,X@args:$1:sort, X^sort)
;

// Wenn X^ eine PP als Argument nimmt,
// dann nicht X@ berprfen, sondern Y@
{X!SYN/Y!SYN} : c$1_subsume_prepsort : 0.3 :
  spec(X^id) & spec(Y^id)
  & X.label=C$1
  & Y.label = C1  
  & subsumes(Kategorien, Adposition, Y^cat)
  & exists(X^args:$1:prepsort) 
  & exists(Y@sort)
  -> 
  subsumes(Sorten, X^args:$1:prepsort, Y@sort)
;

')

Sortencheck(1)
Sortencheck(2)
Sortencheck(3)
Sortencheck(4)


/* --------------------------------------------------

Determiner als Komplemente
==========================

Fr Determiner relevante Ebenen sind die SYN-Ebene und 
die OBL1-Ebene, mit der sich ein Nomen einen Determiner 
sucht. Die allgemeinen Kongruenzen sind unter ->Komplemente 
abgehandelt. Hier stehen nur die zustzlichen Anforderungen
an Determiner.

Weitere Referenzen finden sich bei der Behandlung 
der Flexionsklasse der Adjektive.

Determiner modifizieren immer Nomen und werden evt. 
durch Adverbien modifiziert.

-------------------------------------------------- */
// Ein Determiner steht vor seinem Regens.
//
// (`Joseph, lieber Joseph<--C1--mein' ???)
{X!SYN} : art_order : 0.1 :
  X.label = C1
  & subsumes(Kategorien, Determiner, X@cat)
  ->
  X/
;

// Ein Determiner steht seinem Regens nahe.
{X!SYN} : art_distance : [ 5 / [ 5 + abs( distance(X@id, X^id) ) ] ] :
  subsumes(Kategorien, Determiner, X@cat)
  ->
  abs( distance( X@id, X^id ) ) <= 1
;

// Keine Anbindung des Nomen zwischen Determiner und Nomen.
// {X!SYN \/ Y!OBL1} : det_noun_noun_modifier : 0.1 :
//   subsumes(Kategorien, Determiner, X@cat)
//   ->
//   max( Y@to, Y^to ) <= X^from
//   |
//   min( Y@from, Y^from ) >= X^to
//;
// Das wird auch schon von proj1 gefordert.



/* --------------------------------------------------

Abtrennbare Verbzustze
=======================

Alle abtrennbaren Verbzustze werden als C4 eines finiten Verbs
modelliert, jedenfalls in Hauptsatz- und Fragesatzposition. In
Nebensatzposition wird die Fgung Verb/AVZ als ein Wort modelliert
(wie in der deutschen Orthographie). Daher knnen wir fordern, da ein
AVZ immer rechts steht.

-------------------------------------------------- */
{X/SYN} : AVZ_init : 0.1 :
 X@cat != PTKVZS;


/* --------------------------------------------------

SYN::MOD
========

MOD-Anbindungen knnen zwischen einer Vielzahl von
Worten stehen. Auch die Wurzelanbindung findet mit MOD 
statt. 

-------------------------------------------------- */
{X!SYN} : mod_init : 0.0 :
 X.label = MOD -> (
  nonspec(X^id)
  |
  // 1) adjektivisches Attribut:
  // nchste--MOD-->Woche	
  (X@cat = ADJA & subsumes(Kategorien, Nomen, X^cat)) 
  |
  // 2) Prpositionaladjunkt:
  // im(Bett)--MOD-->schlafen
  // Hammer<--MOD--im(Schrank)
  (subsumes(Kategorien, Adposition, X@cat)
   & (subsumes(Kategorien, Verb, X^cat) |
      subsumes(Kategorien, Nominal, X^cat))) 
  |
  // 3) Apposition:
  // Counsellor--MOD-->Troi
  (subsumes(Kategorien, Appositional, X@cat) &
   subsumes(Kategorien, Nomen, X^cat))
  |
  // 4) unterordnende Konjunktion:
  // wenn--MOD-->(ich)anschaue
   (subsumes(Kategorien, KOUS, X@cat) 
    & subsumes(Kategorien, finit, X^cat))
   |
  // 5) Nebensatzverb an Hauptsatzverb oder Hauptsatzadverb:
  // (es)geht<--MOD--(wenn ich)anschaue
  // da(gefllt es mir)<--MOD--(wo ich)bin 
  (subsumes(Kategorien, finit, X@cat) & 
   (subsumes(Kategorien, finit, X^cat) | subsumes(Kategorien, ADV, X^cat)))
  |
  // 6) Adverb an irgendetwas:
  // komme<--MOD--morgen
  // um(neun)<--MOD--so
  // etwas--MOD-->spt
  // auch--MOD-->(der)Fnfte
  // und weitere mehr...
  (subsumes(Kategorien, Adverb, X@cat) 
   &
   ((exists(X@modifies) & match(Kategorien, X@modifies, X^cat) > 0.0)
    |
    (X@cat = ADJP & subsumes(Kategorien, Verb, X^cat)))
   )
  |
  // 7) Relativsatz an Nomen:
  // (der)Termin<--MOD--(den wir abgesprochen)haben
  (subsumes(Kategorien, finit, X@cat) & subsumes(Kategorien, Nominal, X^cat))
  |
  // 8) Zeitausdruck als Adverbial:
  // habe_Zeit<--MOD--(den ganzen)Tag
  (X@cat=NN 
   & exists(X@timeunit)  
   & compatible(Kasus, acc, X@case)
   & subsumes(Kategorien, Verb, X^cat)) 
 );


// Constraints ber alle MODs
// --------------------------

/*

Da wir optionale Argumente gegenber Adjunkten immer 
prferieren wollen, bekommen _alle_ MOD-Anbindungen (auer
die Wurzelanbindung) eine kleine Strafe.

*/

// keine MODs
{X!SYN} : no_mod : 0.99 :
  X.label!=MOD
;

// Ein MOD steht seinem Regens nahe.
//
// Ausnahmen werden fr die Klammerstellung in Nebenstzen gemacht:
// wenn--MOD-->(wir die ganze Sache auf Mai)verschieben ist zwar sehr
// weit getrennt, soll aber deshalb nicht disprferiert werden ---
// schlielich ist Verbletztstellung die Regel und nicht die Ausnahme.
{X!SYN} : mod_distance : [ 10 / [ 10 + abs( distance(X@id, X^id) ) ] ] :
  X.label=MOD
  ->
  (
   spec(X^id) & (
   // da--MOD-->(wir zur Vertragsunterzeichnung noch weitere Sicherheiten)brauchen
   (subsumes(Kategorien, unterordnend, X@cat) & subsumes(Kategorien, finit, X^cat))
   |
   // (ich)sehe<--MOD--(da Ihr euch alle recht vergnglich miteinander)amsiert
   (subsumes(Kategorien, finit, X@cat) & subsumes(Kategorien, finit, X^cat))
   |
   // (der)Termin<--MOD--(den wir neulich schon abgesprochen)haben
   (subsumes(Kategorien, finit, X@cat) & subsumes(Kategorien, Nominal, X^cat))
   |
   // (wir machen es)da<--MOD--(wo Sie es mir nur immer)sagen
   (subsumes(Kategorien, finit, X@cat) & subsumes(Kategorien, ADV, X^cat)))
   |
   // der--MOD-->Termin
   abs( distance( X@id, X^id ) ) <= 1
   )
;

// Die oben gemachten Ausnahmen werden nur ganz schwach bestraft:
{X!SYN} : mod_distance2 : [ 100 / [ 100 + abs( distance(X@id, X^id) ) ] ] :
  X.label=MOD
  &
  (
   nonspec(X^id)
   |
   (subsumes(Kategorien, unterordnend, X@cat) & subsumes(Kategorien, finit, X^cat))
   |
   (subsumes(Kategorien, finit, X@cat) & subsumes(Kategorien, finit, X^cat))
   |
   (subsumes(Kategorien, finit, X@cat) & subsumes(Kategorien, Nominal, X^cat))
   |
   (subsumes(Kategorien, finit, X@cat) & subsumes(Kategorien, ADV, X^cat))
  )
  ->
  abs( distance( X@id, X^id ) ) <= 1
;

// Zwischen ^ und @ mu zumindest der loctype stimmen
// Das gilt fr Komplemente wie fr Adjunkte.
{X!SYN} : loctype_agree : 0.1 :
 spec(X^id)
 & exists(X^loctype) 
 & exists(X@loctype) 
// & subsumes(Sorten, local, X@sort)
 ->
 X@loctype = X^loctype
;

// Einige MODs stehen vorzugsweise links oder rechts
{X\SYN} : mod_right : [ X@rightpenalty ] :
  X.label=MOD 
  -> ~exists(X@rightpenalty)
;
{X/SYN} : mod_left : [ X@leftpenalty ] :
  X.label=MOD 
  -> ~exists(X@leftpenalty)
;


// Anbindung nach links ist prferiert
{X/SYN} : mod_dir : 0.99 :
 X.label!=MOD
;

/* --------------------------------------------------

Fall 1)

Attributierende Adjektive
=========================

Fr attributierende Adjektive sind zwei Ebenen von Bedeutung. 

Zum einen natrlich die SYN-Ebene, auf der die attributierenden Adjektive
per MOD an das Nomen angebunden werden. Sie kongruieren mit dem
Nomen in Genus, Kasus und Numerus. Ihre Flexionsklasse (stark, gemischt
oder schwach) wird durch den das Nomen modifizierende Determiner bestimmt.

Auf der OBL1-Ebene sucht sich das Nomen seinen Determiner, so da dort 
einige Bedingungen gedoppelt werden mssen.

-------------------------------------------------- */

// Adjektiv steht links vom Nomen.
// (Aber es steht rechts vom Pronomen: `nichts greres'! 
// Daher der Test auf label=MOD.)
{X!SYN} : adj_seq : 0.1 :
  X.label = MOD
  & X@cat = ADJA
  ->
  X/
;

// Genuskongruenz zum Nomen
{X!SYN} : adj_gender : 0.11 :
  spec(X^id)
  & X@cat = ADJA
  & X.label = MOD
  ->
  subsumes(Gender, X^gender, X@gender)
;

// Kasuskongruenz zum Nomen
{X!SYN} : adj_case : 0.12 :
  spec(X^id)
  & X@cat = ADJA
  & X.label = MOD
  ->
  subsumes(Kasus, X^case, X@case)
;

// Numeruskongruenz zum Nomen
{X!SYN} : adj_number : 0.13 :
  spec(X^id)
  & X@cat = ADJA
  & X.label = MOD
  ->
  X^number=X@number
;

// Determiner steht vor Adjektiv
{X!SYN/\Y!SYN} : art_adj_seq : 0.1 :
  X.label=C1
  & Y@cat=ADJA
  ->
  X@to<=Y@from
;
{X!OBL1 / Y!SYN} : obl1_art_adj_seq : redundant : 0.1 :
  Y.label = MOD
  & Y@cat=ADJA
  ->
  X^to<=Y@from
;

// Genuskongruenz Determiner und Adjektiv
{X!SYN/\Y!SYN} : art_adj_gender : 0.12 :
  spec(X^id)
  & exists(X@gender)
  & exists(Y@gender)
  & subsumes(Kategorien, Nomen, X^cat)
  & X.label=C1
  & X@cat != CARD
  & Y@cat=ADJA
  ->
  subsumes(Gender, Y@gender, X@gender)
;
{X!OBL1 / Y!SYN} : obl1_art_adj_gender : redundant : 0.12 :
  spec(X^id)
  & exists(X^gender)
  & exists(Y@gender)
  & subsumes(Kategorien, Nomen, X@cat)	       
  & Y@cat=ADJA
  & X^cat != CARD
  ->
  subsumes(Gender, Y@gender, X^gender)
;

// Kasuskongruenz Determiner und Adjektiv
{X!SYN/\Y!SYN} : art_adj_case : 0.12 :
  spec(X^id)
  & exists(X@case)
  & exists(Y@case)
  & subsumes(Kategorien, Nomen, X^cat)
  & X.label=C1
  & Y@cat=ADJA
  ->
  subsumes(Kasus, Y@case, X@case)
;
{X!OBL1 / Y!SYN} : obl1_art_adj_case : redundant : 0.12 :
  spec(X^id)
  & exists(X^case)
  & exists(Y@case)
  & subsumes(Kategorien, Nomen, X@cat)	       
  & Y@cat=ADJA
  ->
  subsumes(Kasus, X^case, Y@case)
;

// Numeruskongruenz Determiner und Adjektiv
{X!SYN/\Y!SYN} : art_adj_number : 0.12 :
  spec(X^id)
  & subsumes(Kategorien, Nomen, X^cat)
  & X.label=C1
  & Y@cat=ADJA
  ->
  Y@number=X@number
;
{X!OBL1 / Y!SYN} : obl1_art_adj_number : redundant : 0.12 :
  spec(X^id)
  & subsumes(Kategorien, Nomen, X@cat)	       
  & Y@cat=ADJA
  ->
  X^number=Y@number
;

// Ein Adjektiv steht seinem Regens nahe.
{X!SYN} : adj_distance : [ 1 / abs( distance(X@id, X^id) ) ] :
  X@cat=ADJA
  ->
  distance(X@id, X^id) <= 1
;

// wenn definiter Determiner, dann schwach
{X!SYN/\Y!SYN} : art_adj_def_schwach : 0.2 :
  Y@cat=ADJA
  & subsumes(Kategorien, Determiner, X@cat)
  & X@definite=yes
  ->
  Y@flexion=weak
;
{X!OBL1 / Y!SYN} : obl1_art_adj_def_schwach : redundant : 0.2 :
  spec(X^id)
  & Y@cat=ADJA
  & subsumes(Kategorien, Determiner, X^cat)
  & X^definite=yes
  ->
  Y@flexion=weak
;

// wenn definiter Determiner, dann nicht stark
{X!SYN/\Y!SYN} : art_adj_def_nicht_stark : 0.8 :
  Y@cat=ADJA
  & subsumes(Kategorien, Determiner, X@cat) & X@definite=yes
  ->
  Y@flexion!=strong
;
{X!OBL1 / Y!SYN} : obl1_art_adj_def_nicht_stark : redundant : 0.8 :
  spec(X^id)
  & Y@cat=ADJA
  & subsumes(Kategorien, Determiner, X^cat) & X^definite=yes
  ->
  Y@flexion!=strong
;

// wenn indefiniter Determiner, dann gemischt
{X!SYN/\Y!SYN} : art_adj_indef_gemischt : 0.2 :
  Y@cat=ADJA
  & subsumes(Kategorien, Determiner, X@cat) & X@definite=no
  ->
  Y@flexion=mixed
;
{X!OBL1 / Y!SYN} : obl1_art_adj_indef_gemischt : redundant : 0.2 :
  spec(X^id)
  & Y@cat=ADJA
  & subsumes(Kategorien, Determiner, X^cat) & X^definite=no
  ->
  Y@flexion=mixed
;

// wenn kein Determiner, dann stark
{X|OBL1 / Y!SYN} : obl1_adj_no_art_strong : 0.1 :
  Y@cat=ADJA
  ->
  Y@flexion=strong
;

// wenn kein Determiner, dann nicht schwach
{X|OBL1 / Y!SYN} : obl1_adj_no_art_not_weak : 0.8 :
  Y@cat=ADJA
  ->
  Y@flexion!=weak
;

// gleiche Deklination fr parallele Adjektive
{X!SYN /\ Y!SYN} : adj_parallel_same_flexion : 0.2 :
    X@cat=ADJA
  & Y@cat=ADJA
  -> 
  X@flexion=Y@flexion
;

/* --------------------------------------------------

Fall 2)

Prpositionen
=============

Fr Prpositionen relevante Ebenen sind die SYN-Ebene und 
die OBL1-Ebene, mit der sich die Prposition ein Nomen
sucht. Die allgemeinen Kongruenzen sind unter ->Komplemente 
abgehandelt. Hier stehen nur die zustzlichen Anforderungen
an PPs.

-------------------------------------------------- */
// Ein Nomen einer PP steht hinter seinem Regens.
{X!SYN} : pp_order : 0.1 :
  spec(X^id)
  & X^cat=APPR
  & subsumes(Kategorien, Nominal, X@cat)
  & X.label=C1
  ->
  X\
;
{X!OBL1} : obl1_pp_order : redundant : 0.1 :
  X@cat=APPR
  ->
  X/
;

// Und dasselbe fr Postpositionen
{X!SYN} : post_order : 0.1 :
  spec(X^id)
  & X^cat=APPO
  & subsumes(Kategorien, Nominal, X@cat)
  & X.label=C1
  ->
  X/
;
{X!OBL1} : obl1_post_order : redundant : 0.1 :
  X@cat=APPO
  ->
  X\
;

// Ein Nomen einer PP steht seinem Regens nahe.
{X!SYN} : pp_distance : [ 10 / [ 10 + abs( distance(X@id, X^id) ) ] ] :
  (nonspec(X^id) | subsumes(Kategorien, Adposition, X^cat))
  & subsumes(Kategorien, Nominal, X@cat)
  & X.label=C1
  ->
  abs( distance( X@id, X^id ) ) <= 1
;
{X!OBL1} : obl1_pp_distance : redundant : [ 10 / [ 10 + abs(distance(X@id, X^id) ) ] ] :
  subsumes(Kategorien, Adposition, X@cat)
  ->
  abs( distance( X@id, X^id ) ) <= 1
;

// Prpositionen werden bevorzugt nach links angebunden.
{X!SYN} : p_modifies_left : 0.9 :
  X.label = MOD
  & subsumes(Kategorien, Adposition, X@cat)
  ->
  X\
;

// Prpositionen knnen nicht an das direkt rechts von ihnen
// stehende Nomen angebunden werden.
{X!SYN} : p_not_immediately_to_right : 0.1 :
  subsumes(Kategorien, APPR, X@cat)
  ->
  X@to!=X^from
;
// ... auch nicht, wenn noch der entsprechende Determiner 
// dazwischen steht.
{X!SYN /\ Y!SYN} : p_not_almost_immediately_to_right : 0.1 :
  subsumes(Kategorien, APPR, X@cat)
  & subsumes(Kategorien, Determiner, Y@cat)
  ->  
  X@to!=Y@from
  |
  Y@to!=Y^from
;


// PPs drfen zwar Pronomen modifizieren (`das mit den Bienen und Vgeln'),
// nicht aber PPER (`ich mit meinem Ordnungstick...'), sondern
// eigentlich nur Demonstrativpronomen.
{X!SYN} : PP_with_pronoun : 0.1 :
 spec(X^id)
 & subsumes(Kategorien, Adposition, X@cat) 
 & subsumes(Kategorien, Nominal, X^cat)
 ->
 X^cat = NN 
 |
 subsumes(Kategorien, Demonstrativpronomen, X^cat)
;

// ... aber noch lieber modifizieren sie Verben.
{X!SYN} : PP_with_Nominal : 0.9 :
 spec(X^id)
 & subsumes(Kategorien, Adposition, X@cat) 
 ->
 subsumes(Kategorien, Verb, X^cat)
;

/* --------------------------------------------------

Fall 3)

Appositionen
============

Appositionen sind meistens postmodifizierende, nominale 
Nomenmodifikatoren.

-------------------------------------------------- */
// Eine Apposition steht hinter ihrem Regens.
{X!SYN} : appo_order : 0.1 :
  spec(X^id)
  & X^cat=NN
  & subsumes(Kategorien, Nominal, X@cat)
  & X.label=MOD
  ->
  X\
;

// Eine Apposition kongruiert im Kasus (manchmal) oder 
// steht im Nominativ. 
// Ausnahme: Datumsausdrcke knnen immer im Akkusativ stehen.
// Zweite Ausnahme: Genitivattribute unterliegen keiner Kongruenz.
{X!SYN} : appo_agree_case : 0.8 :
  spec(X^id)
  & exists(X@sort)
  & X^cat=NN
  & subsumes(Kategorien, Nominal, X@cat)
  & X.label=MOD
  & ~subsumes(Kasus, gen, X@case)
  ->
  ( 
    subsumes(Kasus, X@case, X^case)
  |
    subsumes(Kasus, X^case, X@case)
  |
    ( subsumes( Sorten, temporal, X@sort ) -> subsumes(Kasus, acc, X@case) )
  )
;
{X!SYN} : appo_case_nom : 0.8 :
  spec(X^id)
  & X^cat=NN 
  & X@case!=X^case
  & subsumes(Kategorien, Nominal, X@cat)
  & X.label=MOD
  & ~subsumes(Kasus, gen, X@case)
  ->
  ( 
    subsumes(Kasus, nom, X@case)
  |
    ( subsumes( Sorten, temporal, X@sort ) -> subsumes(Kasus, acc, X@case) )
  )
;

// Eine Apposition kongruiert im Numerus (meistens).
// Stimmt das berhaupt?
{X!SYN} : appo_agree_number : 0.4 :
  spec(X^id)
  & X^cat=NN
  & subsumes(Kategorien, Nominal, X@cat)
  & X.label=MOD
  & ~subsumes(Kasus, gen, X@case)
  ->
  X@number=X^number
;

/* --------------------------------------------------

Fall 4)

Unterordnungen
==============

-------------------------------------------------- */

// die unterordnende Konjunktion steht links.
{X!SYN} : junction_order : 0.0 :
 spec(X^id)
 & subsumes(Kategorien, unterordnend, X@cat) 
 & subsumes(Kategorien, finit, X^cat)
 ->
 X/
;
// X@cat kann auch ein Relativpronomen sein,
// das C1 statt MOD ist. Daher kein Test auf X.label.

// Fr die Stellung der beiden finiten Verben gilt das nicht:
// ich komme, wenn es geht --- wenn es geht, komme ich

// Also keine weiteren order-Constraints fr Nebenstze.


/* --------------------------------------------------

Fall 5)

Verb an Verb
============

Dieser Fall wird durch Constraints der Ebene SUB eingeschrnkt.

-------------------------------------------------- */


// Constraints ber Fall 6)
// ------------------------

// Adverben modifizieren alles mgliche
{X!SYN} : ADV_match : [ match( Kategorien, X@modifies, X^cat ) ] :
  spec(X^id)
  & X.label = MOD
  & subsumes(Kategorien, Adverbial, X@cat)
  & exists(X@modifies)
  -> 
  match( Kategorien, X@modifies, X^cat ) = 1.0;

// Aber Adverben modifizieren nicht die Komplemente von Prpositionen
// (`bei<--C1--Ihnen<--MOD--auch'? 
// Nein, lieber `bei(Ihnen)<--MOD--auch'.)
{X!SYN/Y!SYN} : ADV_PP : 0.0 :
 spec(X^id)
 & X.label=C1 & subsumes(Kategorien, Adposition, X^cat)
 & 
 Y.label=MOD & subsumes(Kategorien, Adverb, Y@cat)
 ->
 false
;

// Adverben, die Adverben modifizieren, gehen nach rechts
{X!SYN} : ADV_right : 0.1 :
 X.label=MOD
 & subsumes(Kategorien, Adverbial, X@cat) 
 & (nonspec(X^id) | subsumes(Kategorien, Adverbial, X^cat))
 ->
 X/
;


// `als' ist ein Spezialfall: es regiert nicht einen festen Kasus,
// sondern den Kasus des dominierenden Wortes.
{X!SYN\Y:SYN} : als_agree_case : 0.1 :
 spec(X^id)
 & X^word = als 
 & X.label = MOD
 & exists(Y^case)
 ->
 X@case = Y^case
;


/* --------------------------------------------------

Fall 7)

Relativstze
============

keine weiteren Constraints, alle Bedingungen
werden auf der Ebene SUB gestellt.

-------------------------------------------------- */


/* --------------------------------------------------

Fall 8)

Accusativus absolutus
=====================

Keine weiteren Bedingungen, dieser Fall ist hoffentlich 
durch die restriktive Sorte genug eingeschrnkt.

-------------------------------------------------- */


/* --------------------------------------------------

Nebenstze
==========

a)

Relativstze hngen vom modifizierten Nomen ab:

(der)Termin<----(den)(wir)machen

da das Relativpronomen nicht immer nach oben gestellt werden 
kann.

Das finite Verb darf aber nur unter einem Nomen stehen,
das mit einem Relativpronomenaufgegriffen wurde. 
Dieses Referieren wird auf der Ebene SUB modelliert:

SUB:  Termin---->den_rel
===

OBL4: den_rel---->Termin
====

b)

In Konjunktionsnebenstzen dagegen hngt finitum von Verb ab.

(wie)wre(es)<--MOD--(wenn)(wir)(uns)treffen

Auch das ist nur eingeschrnkt erlaubt: nmlich dann, wenn
das untere Verb eine KOUS vorweisen kann. Auch diese Vorweisung
geschieht auf der SUB-Ebene:

SUB:  wenn<----treffen
===

-------------------------------------------------- */

// Die einleitende Konjunktion ist eine Konjunktion.
{X!SUB} : sub_init : 0.0 :
 (subsumes(Kategorien, finit, X@cat) 
  & (nonspec(X^id) | subsumes(Kategorien, unterordnend, X^cat)))
 |
 (subsumes(Kategorien, Relativantezedens, X@cat) 
  & (nonspec(X^id) | subsumes(Kategorien, Relativpronomen, X^cat)))
;

// Die einleitende Konjunktion ist einleitend.
{X/SUB} : sub_left : 0.0 : 
  spec(X^id)
  & subsumes(Kategorien, finit, X@cat) 
  & subsumes(Kategorien, unterordnend, X^cat)
  -> 
  false
;
 
// wenn das Verb auf eine Konjunktion zeigt, dann
// darf es nicht Satzwurzel sein.
{X|SYN\/Y!SUB} : dangling_subclause : 0.1 : 
 subsumes(Kategorien, Verb, X@cat) -> false;
// Wenn hingegen X@cat = Noun, wie etwa in `The house that Jack
// built.', dann wird ja schon die Kante house--MOD-->NIL 
// bestraft. Deshalb wird dieser Fall hier nicht nochmals bestraft. 



// Relativpronomen:
// ================

// PREL kongruieren im Numerus
{X!SUB} : rel_number : 0.1 :
 spec(X^id)
 & X^cat = PREL & subsumes(Kategorien, Nominal, X@cat) 
 -> 
 X^number=X@number
;

// PREL kongruieren im Genus
{X!SUB} : rel_gender : 0.1 :
 spec(X^id)
 & X^cat = PREL & subsumes(Kategorien, Nominal, X@cat) 
 -> 
 compatible(Gender, X^gender,X@gender)
;

// Bei den Relativadverben sollten hnliche Bedingungen gestellt werden:
//
// Der Ort, wo wir uns treffen...
// *Die Zeit, wo wir uns treffen...
// 
// Die Qualitt, wie die Kunden sie erwarten...
// 

// Das ist aber schwierig, weil die Verbmobildomne konsequent 
// Zeit- und Ortsangaben mischt.


// Relativpronomen sind anaphorisch
{X\SUB} : rel_anaphoric : 0.1 : 
  subsumes(Kategorien, Nominal, X@cat) -> false;

// zuerst errichten wir ein Schlupfloch in der SYN-OBL4-Kopplung:
{X:SYN / Y!OBL4} : c4_obl4_coupling2 : 0.0 :
  (X.label=C4 & X\Y)
  |
  subsumes(Kategorien, Relativpronomen, Y@cat)
;

// Dieses Constraint ersetzt das zuvor automatisch generierte
// Constraint gleichen Namens. Daher ist es STRENGSTENS VERBOTEN,
// DIESEN BLOCK AN EINE ANDERE STELLE DER DATEI ZU VERSCHIEBEN.

// Dann stellen wir Bedingungen an das Komplement eines Relativpronomens:

// Die Verbindung [SUB:]N---->PREL mu von OBL4 gespiegelt werden
{X!SUB \ Y:OBL4} : sub_obl4_coupling1 : 0.0 :
  spec(X^id)
  & subsumes(Kategorien, Relativpronomen, X^cat)
  & subsumes(Kategorien, Nominal, X@cat)
  ->
  X/Y
;

// Die Verbindung [OBL4:]PREL---->N mu von SUB gespiegelt werden
{X!OBL4 \ Y:SUB} : obl4_sub_coupling1 : 0.0 :
  spec(X^id)
  & subsumes(Kategorien, Relativpronomen, X@cat)
  & subsumes(Kategorien, Nominal, X^cat)
  ->
  X/Y
;

// Obwohl es eine C4-Stellung vergibt, darf das Relativpronomen
// keine syntaktischen Modifikanden haben.
{X!SYN} : rel_init : 0.0 :
  spec(X^id)
  & subsumes(Kategorien, Relativpronomen, X^cat)
  -> 
  false
 ;

// Ein Relativpronomen steht unmittelbar vor dem Nebensatzsubjekt:
// `Der Tag, an dem wir es machen'
// oder ist selbst Nebensatzsubjekt.
// Das ist natrlich nur eine ungefhre Beschreibung des Phnomens:
// `Der Tag, den die Gesprchsteilnehmer...'
// Aber wir brauchen dieses Constraint, um z.B. in n004k030-2 zu
// disambiguieren: 
// `...da wir sowas auch in diesem Monat in dem Mai vormachen'
// Hier ist sonst nicht auszuschlieen, da `in dem Mai
// vormachen' als Relativsatz interpretiert wird. Um dem abzuhelfen,
// fordern wir, da das Wort nach einem Relativpronomen als Subjekt
// gebunden wird.
{X:SYN,Y:SYN} : rel_ifo_SUBJ : 0.5 :
 subsumes(Kategorien, Relativpronomen, X@cat)
 & X@to = Y@from
 ->
 (Y.label = C1 & (nonspec(Y^id) | subsumes(Kategorien, finit, Y^cat)))
 |
 (X.label=C1 & (nonspec(X^id) | subsumes(Kategorien, finit, X^cat)))
;


// Unterordnungen:
// ===============

// wenn das Verb auf eine Konjunktion zeigt,
// dann zeigt diese syntaktisch auf das Verb.
//
// Ausnahmen mssen wir fr Relativpronomen machen,
// denn diese knnen weiter weg vom finiten Verb stehen:
// ...den--C1-->abgesprochen--C1-->(wir)haben
// ...(mit<--C1--dem)--MOD-->(wir)fahren
{X!SUB\Y:SYN} : KOUS_sub_syn_coupling : 0.0 :
 spec(X^id)
 & subsumes(Kategorien, finit, X@cat) 
 & subsumes(Kategorien, unterordnend, X^cat) 
 ->
 subsumes(Kategorien, Relativpronomen, X^cat)
 |
 X/Y
; 
// Wenn eine Konjunktion auf das Verb zeigt,
// dann zeigt es auf der SUB-Ebene zurck auf die Konjunktion.
{X/SYN\Y:SUB} : KOUS_syn_sub_coupling : 0.0 :
 spec(X^id)
 & subsumes(Kategorien, unterordnend, X@cat) 
 & subsumes(Kategorien, finit, X^cat) 
 ->
 X/Y
; 

// Damit Relativpronomen trotz Einschachtelung auf das richtige Verb
// zeigen, stellen wir hier noch weitere Bedingungen:
// a) Relativpronomen zeigen syntaktisch nach rechts oder auf
//    Prpositionen
{X!SYN} : PREL_pref1 : 0.1 :
 subsumes(Kategorien, Relativpronomen, X@cat)
 ->
 X/ | (spec(X^id) & X^cat = APPR)
;

// Wenn ein Relativpronomen Komplement zu einer APPR ist, dann soll
// wenigstens diese nach rechts zeigen.
{X!SYN\Y!SYN} : PREL_pref2 : 0.1 :
 spec(X^id)
 & subsumes(Kategorien, Relativpronomen, X@cat)
 & X^cat = APPR
 ->
 Y/
;
 


// wenn eine Konjunktion auf das Verb zeigt, 
// dann steht sie zuerst im Satz...
{X!SYN/\Y!SYN} : KOUS_erststellung : 0.0 :
 subsumes(Kategorien, unterordnend, X@cat)
 ->
 distance(X@id, Y@id) > 0
; 

// ...und das Verb als letztes
{X!SYN/\Y\SYN} : verbletztstellung : 0.0 :
 subsumes(Kategorien, unterordnend, X@cat)
 ->
 false
; 

// Disambiguierung: es gibt das adverbiale `wie' 
// und das unterordnende `wie'. Das adverbiale `wie'
// darf nicht zugleich mit anderen MODs vor dem finiten Verb stehen.
// Dasselbe gilt fr alle PWAVS.
{X/SYN/\Y/SYN} : PWAV_KOUS_disambiguation : 0.0 :
 spec(X^id)
 & subsumes(Kategorien, finit, X^cat)
 & subsumes(Kategorien, PWAV, X@cat)
 ->
 false
;
 
// nur ein PREL lizensiert die Kante V--MOD-->Nominal.
{X:SUB/Y!SYN} : v_mod_n : 0.0 :
 spec(X^id) & spec(Y^id)
 & Y.label = MOD
 & subsumes(Kategorien, finit, Y@cat)
 & subsumes(Kategorien, Nominal, Y^cat)
 ->
 X! & subsumes(Kategorien, Relativpronomen, X^cat)
;

// nur eine KOUS lizensiert die Kante V--MOD-->V.
{X:SUB\/Y!SYN} : v_mod_v : 0.0 :
 spec(X^id) & spec(Y^id)
 & Y.label = MOD
 & subsumes(Kategorien, finit, Y@cat)
 & subsumes(Kategorien, Verb, Y^cat)
 ->
 ( X! & subsumes(Kategorien, KOUS, X^cat))
 |
 start(X@id)
;
// Der letzte Fall ist fr konjunktionslose Nebenstze: `Soll die Frau
// wie Feuer werden, mu der Mann wie Eis sein.'  Der Sinn ist
// derselbe wie in `Wenn die Frau...', aber im Nebensatz herrscht
// Verberststellung. Einen Teil dieser Stze kann man daran erkennen,
// da X@ das erste Wort ist. Aber was mit `Ich wre glcklicher,
// htte ich geschwiegen.' oder mit `Denn willst du tausend Schritte
// laufen, mut du Jgermeister saufen.'? Hoffen wir, da wir so was
// nicht brauchen.
//
// All das gilt nur fr Adjunkte. Komplemente drfen auch ohne
// Konjunktion stehen: `ich denke, das reicht.' Daher die Prfung auf 
// label = MOD.


// Die Kante V--MOD-->ADV ist noch strker eingeschrnkt.
// Es mu eine KOUS beteiligt sein, und sie mu sortal passen:
//
// Erlaubt:
//
// X: wenn<----(es)geht
// Y:              geht----MOD---->dann
//                                 dann----MOD---->machen(wir es)
// Verboten:
//
// X: wenn<----(es)geht
// Y:              geht----MOD---->dort
//                                 dort----MOD---->machen(wir es)
{X:SUB\/Y!SYN} : v_mod_a : 0.0 :
 spec(X^id) & spec(Y^id)
 & Y.label = MOD
 & subsumes(Kategorien, finit, Y@cat)
 & subsumes(Kategorien, ADV, Y^cat)
 ->
 X! & (
    ( Y^word = so & X^word = da ) |
    ( Y^word = dann & X^word = wenn ) |
    ( Y^word = dort & X^word = wo ) |
    ( Y^word = da & X^word = wo ) 
 )
;

/* --------------------------------------------------

Nebenordnung von Hauptstzen durch KON
======================================

Konjunktion von Hauptstzen ist modelliert als
das Belegen von zwei Valenzen der Konjunktion.

-------------------------------------------------- */

// Wenn eine KON zwei Hauptstze verbindet, 
// dann mssen das auch richtige Hauptstze
// sein, d.h. Vorfeld ja, SUB nein.
{X!SYN\/Y|VORFELD} : KON_VORFELD : 0.0 :
 spec(X^id)
 & X^cat = KON
 & subsumes(Kategorien, finit, X@cat)
 & X.label != MOD
 -> 
 false 
;

{X!SYN\/Y!SUB} : KON_SUB : 0.0 :
 spec(X^id)
 & X^cat = KON
 & X.label != MOD
 -> 
 false 
;


// Das erste Argumente der KON ist optional, 
// aber es darf nur fehlen, wenn die KON vorn im Satz steht
{X|OBL1} : KON_OBL1 : 0.0 :
 X@cat = KON
 ->
 start(X@id)
;

/* --------------------------------------------------

Thematische Rollen: AGENT, EXP, PATIENT, QUALITY, THEME
=======================================================


-------------------------------------------------- */

m4_define(M_tr_init,`m4_dnl
// existierende thematische Rolle mu von X@ lizensiert werden
{X!$1} : $1_init : 0.0 :
  X@args:1:role=$1
  | X@args:2:role=$1
  | X@args:3:role=$1
  | X@args:4:role=$1
;
// eine lizensierte Rolle soll existieren
{X|$1} : $1_no_root : 0.8 :
    (exists(X@args:1:role) -> X@args:1:role!=$1)
  & (exists(X@args:2:role) -> X@args:2:role!=$1)
  & (exists(X@args:3:role) -> X@args:3:role!=$1)
  & (exists(X@args:4:role) -> X@args:4:role!=$1)
;

// thematische Kante darf nicht parallel zur Syntax gehen
//
// z.B. soll das hier nicht passieren:
//
//  [SYN:]habe<--C2--gehrt
//  [THEME:]habe<--MOD--gehrt
//
{X!$1==Y!SYN} : $1_SYN_cycle : 0.0 : 
  false;

// thematische Kante darf nicht an das Komplement einer Adposition gehen
//
// z.B. soll das hier nicht passieren:
// 
//  [SYN:]bei<--C1--Ihnen
//  [THEME:]geht---->Ihnen
//
{X!$1\Y!SYN} : $1_under_prep : 0.0 : 
  spec(Y^id)
  & Y.label = C1 
  & subsumes(Kategorien, Adposition,Y^cat)
  ->
  false
;

// KON teilt die Stze auch thematisch, d. h. keine 
// thematische Kante darf die Konjunktion "kreuzen".
// n005k005-1:
// DATUM wrde mir passen aber es mu vormittags sein.
//                   |         ^
//                   +---------+ THEME
// falsch!
{X|SYN,Y!$1} : KON_$1_no_crossing : 0.0 :
 X@cat = KON
 & min(Y@from, Y^from) < X@from
 & max(Y@to, Y^to) > X@to
 ->
 false
;

// Thematische Rollen werden im Nebensatz gesttigt.
// SUB geht im Nebensatz immer nach links.
{X!SUB \/ Y!$1} : SUB_$1_no_crossing : 0.0 :
 subsumes( Kategorien, finit, X@cat) 
 -> 
 X^from <= Y^from
 & Y\
;
// Aber auch: 
//   Ich sage, da ich ihm das Buch geben sollte.
// ternr!
// Wird z. T. durch Projektivitt (syntaktisch, 
// aber nicht semantisch) abgedeckt.

// Damit ist nur ein Teil der Einschrnkungen formuliert.
// Weitere wrden z. B. Modalstze betreffen:
//  Sollte ich nicht dasein warten Sie bitte.


// Wenn eine thematische Kante von einem Partizip zu
// einem Wort geht, das als Subjekt unter einem
// VAUXFIN hngt, dann nehmen wir mal an, da es die 
// thematische Rolle des ersten Arguments sein mu.
{X!$1 \ Y!SYN} : sein_aux_perfect_sorts_$1 : 0.2 :
  spec(Y^id)
  & Y^cat=VAUXFIN
  & Y.label=C1
  & X@cat=PPP
  ->
  X@args:1:role= $1
;

')

m4_define(M_tr_sort2,`m4_dnl
// thematische Rolle soll die Sortenforderung erfllen
// (auer es ist eine PP --> das wird weiter unten behandelt)
{X!$1} : $1_$2_sort : 0.3 :
  spec(X^id)
  & X@args:$2:role=$1 
  & exists(X@args:$2:sort)
  & ~subsumes(Kategorien, Adposition, X^cat)
  -> 
  exists(X^sort) 
  & subsumes(Sorten, X@args:$2:sort, X^sort)
;

// Wenn X^ eine PP ist, 
// dann nicht X^ berprfen, sondern Y@
{X!$1/\Y!SYN} : $1_$2_prepsort : 0.3 :
  X@args:$2:role=$1 
  & Y.label=C1
  & (nonspec(Y^id) | subsumes(Kategorien, Adposition, Y^cat))
  & exists(X@args:$2:prepsort)
  -> 
  exists(Y@sort) 
  & subsumes(Sorten,X@args:$2:prepsort,Y@sort)
;

// Wenn X^ nicht ausdrcklich eine PP sein kann
// (durch Angabe von args:n:prepsort),
// dann darf es keine sein.
{X!$1} : $1_$2_PP_forbidden : 0.0 :
  spec(X^id)
  & X@args:$2:role=$1 
  & ~exists(X@args:$2:prepsort)
  ->
  ~subsumes(Kategorien, Adposition, X^cat)
;

')


m4_define(M_tr_obl_coup2,`m4_dnl
// genau wie die SYN-OBL-Kopplung
{X!$1 == Y!OBL$2} : $1_OBL$2_parallel1 : 0.1 :
  Y@args:$2:role=$1
;

{X:$1 \/ Y!OBL$2} : $1_OBL$2_parallel2 : 0.1 :
  Y@args:$2:role=$1
  ->
  X==Y
;

{X!$1 \/ Y|OBL$2} : $1_OBL$2_obligatory : 0.1 :
  Y@args:$2:role=$1
  & Y@args:$2:realized = yes
  ->
  false
;

')
m4_define(M_tr_obl_coupling,`m4_dnl
M_tr_obl_coup2($1,1)
M_tr_obl_coup2($1,2)
M_tr_obl_coup2($1,3)
M_tr_obl_coup2($1,4)
')

m4_define(M_tr_Sortencheck,`m4_dnl
M_tr_sort2($1,1)
M_tr_sort2($1,2)
M_tr_sort2($1,3)
M_tr_sort2($1,4)

// Wenistens soll berhaupt eine Sorte existieren -> Inhaltswort oder PP
{X!$1} : $1_sort_exist : 0.0 :
  spec(X^id)
  & (subsumes(Kategorien, Adposition, X^cat) | exists(X^sort))
;

')

m4_define(M_tr_Distance, `m4_dnl
// Wenn die Theta-Rolle nicht parallel mit dem Komplement geht, dann
// soll sie als letztes Mittel nach der Entfernung ausgewhlt werden
{X!$1} : $1_distance : [ 1000 / [ 1000 + abs( distance(X@id, X^id) ) ] ] :
  abs( distance( X@id, X^id ) ) <= 1
;
')

m4_define(M_tr_common,`m4_dnl
M_tr_init($1)
M_tr_Sortencheck($1)
M_tr_Distance($1)
M_tr_obl_coupling($1)
')

M_tr_common(AGENT)
M_tr_common(EXP)
M_tr_common(PATIENT)
M_tr_common(QUALITY)
M_tr_common(THEME)



/* --------------------------------------------------

Vorfeldbesetzung
================

Finite Verben zeigen im V2-Satz auf der VORFELD-Ebene
auf das das Vorfeld besetzende Element. 

Beispiel:

  Der Mann schlft.
        ^     |
        +-----+

Merke:

   o______________
                  \ C1
               ____o       SYN
           C1 /
           __o
       C1 /
         o
  Aber der Mann schlft.
            ^      |
            +------+       VORFELD




  Ich schlafe weil der Mann schlft.
   ^      |                     |
   +------+                     v

  Lassen Sie uns einen Termin ausmachen.
     |
     v

-------------------------------------------------- */
// Nur finite Verben haben ein VORFELD.
{X!VORFELD} : vorfeld_init : 0.0 :
  subsumes(Kategorien, finit, X@cat)
;

// VORFELD steht links
{X/VORFELD} : vorfeld_vorne : 0.0 : 
  false
;

// nur ein Verb pro VORFELD
{X!VORFELD /\ Y!VORFELD} : vorfeld_uniq : 0.0 : 
  false
;

// Im Hauptsatz (auer Verberstsatz) existiert das VORFELD
{X|VORFELD\/Y|SUB} : no_vorfeld : 0.0 :
  // Wenn ein finites Verb im Hauptsatz kein Vorfeld hat, dann mu es 
  subsumes(Kategorien, finit, Y@cat)
  ->
  // am Satzanfang stehen.
   start(X@id)
;

// Im Nebensatz existiert das VORFELD nicht, Teil I
// Nebensatz erkenntlich an existierender Konjunktion
{X!VORFELD\/Y!SUB} : vorfeld_subclause : 0.0 :
  false
;

// Im Nebensatz existiert das VORFELD nicht, Teil II
// Nebensatz direkt erkenntlich an der Verbform
{X!VORFELD} : vorfeld_SVO : 0.0 :
  exists(X@pos) -> X@pos = SVO
;

// Wenn es vor & unter dem Verb steht,
// mu es VORFELD sein
{X/SYN \ Y!VORFELD} : under_verb_vorfeld : 0.1 :
  X/Y
;

// Wenn es vor & unter dem Verb steht,
// mu es VORFELD sein, aber wenn es von dieser Sorte
// mehr als eins gibt, dann nimm das vorne stehende.
{X/SYN \ Y!VORFELD} : not_under_verb_vorfeld : 0.5 :
  ~ X/Y
  ->
  Y^from < X@from
;

// VORFELD hngt unterm Verb 
// oder unter einem Teil eines (hoffentlich desselben) Verbs
{X!SYN / Y!VORFELD} : vorfeld_under_verb : 0.0 :
  spec(X^id)
  & (X\Y | (subsumes(Kategorien, Verb, X^cat) & distance(X^id, Y@id) < 0))
;

// Wenn das Vorfeld nicht unter dem Verb hngt,
// soll es unter dem letzten Wort hngen
{X:SYN / Y:VORFELD} : vorfeld_under_last_word : 0.1 :
  ~X\Y
  ->
  stop(X^id)
;


/* --------------------------------------------------

Spezialflle
============

- `noch ein..'

- Kausativ-lassen

-------------------------------------------------- */
// `noch ein..'
{X:SYN, Y:SYN} : noch_indef : 0.1 :
  X@to=Y@from 
  & X@word=noch 
  & Y@cat=ARTINDEF 
  -> 
  X\Y;

// Kausativ-lassen AGENT-Anbindung
{X!PATIENT, Y!AGENT} : kausativ_AGENT_lassen : 0.9 :
  X@baseform=lassen
  & subsumes(Kategorien, infinit, Y@cat)
  ->
  X^id=Y^id
;

// Bestrafung von Demonstrativpronomen, die wie Artikel aussehen
{X:SYN, Y:SYN} : PROS_with_N : 0.1 :
 X@to = Y@from
 & X@cat = PROS
 & (subsumes(Kategorien, Nomen, Y@cat) | subsumes(Kategorien, Adjektiv, Y@cat))
 ->
 false
;

// --------------------------------------------------
// "sein" steht nur sehr allgemein im Lexikon.
// Die folgenden Constraints schrnken die 
// Verwendung ein.
// --------------------------------------------------

{X!SYN} : sein_main_c1 : 0.0 :
  spec(X^id)
  & X^baseform=sein
  & X^cat=FIN
  & X.label=C1
  -> 
  (
    // "das ist mir zu spt."
    subsumes( Kategorien, Nominal, X@cat )
  ) | (
    // "dienstags ist zu spt."
    subsumes(Kategorien, Adverbial, X@cat)
    & exists(X@sort) & subsumes(Sorten, time_or_place, X@sort)
  )
;


{X!SYN} : sein_main_c2_init : 0.0 :
  spec(X^id)
  & X^baseform=sein
  & X.label=C2
  -> 
  (
    // "das ist mir zu spt."
    subsumes(Kategorien, Nominal, X@cat)
    & X@case=dat
  ) | (
    // "das ist bei mir schon voll."
    X@cat=APPR
    & X@case=bei
  ) | (
    // "das ist fr mich zu spt."
    X@cat=APPR
    & X@case=fr
  )
;

{X!SYN/Y!SYN} : sein_main_c2_sort : 0.8 :
  spec(X^id) & spec(Y^id)
  & exists(Y@sort)
  & X^baseform=sein
  & X.label=C2
  & Y.label=C1
  & Y^cat=APPR
  & ( Y^case=bei | Y^case=fr )
  -> 
  subsumes( Sorten, human, Y@sort )
;

{X!SYN} : sein_main_c3_init : 0.0 :
  spec(X^id)
  & X^baseform=sein
  & X.label=C3
  -> 
  (
    // "Karin ist die/eine Lehrerin."
    subsumes(Kategorien, Nominal, X@cat)    
  ) | (
    // "das ist mir zu spt."
    X@cat=PTKZU
  ) | (
    // "das ist in Hamburg." und "Ich bin am Laufen."
    X@cat=APPR
  ) | (
    // "die Konferenz ist morgen."
    X@cat=ADV
  ) | (
    // "das ist gut."
    X@cat=ADJP
  ) | (
    // "Ich bin trainieren."
    X@cat=INF
  ) | (
    // "Die Schlacht ist zu schlagen."
    X@cat=INFZU
  )
;

// Kasusbereinstimmung bei identifizierendem sein/Kopula
{X!SYN} : sein_main_c3_nom : 0.2 :
  spec(X^id)
  & X^baseform=sein
  & X.label=C3
  & subsumes(Kategorien, Nominal, X@cat) 
  -> 
  exists(X@case) &
  subsumes(Kasus, X@case, nom)
;

// Numerusbereinstimmung bei identifizierendem sein/Kopula
{X!SYN} : sein_main_c3_number : 0.5 :
  spec(X^id)
  & X^baseform=sein
  & X.label=C3
  & subsumes(Kategorien, Nomen, X@cat) 
  -> 
  X@number=X^number
;

// ??
{X!SYN} : sein_main_c3_APPR : 0.5 :
  spec(X^id)
  & X^baseform=sein
  & X.label=C3
  & X@cat=APPR
  -> 
  // Ja, was kann man denn fr diese Art fordern?
  true
;

// Adverbiale mit sein sind orts- oder zeitbestimmend
{X!SYN} : sein_main_c3_ADV : 0.8 :
  spec(X^id)
  & X^baseform=sein
  & X.label=C3
  & X@cat=ADV
  -> 
  exists(X@sort)
  & subsumes( Sorten, time_or_place, X@sort )
;

// Verlaufform Sortenbereinstimmung
{X!SYN} : sein_main_c3_INF : 0.5 :
  spec(X^id)
  & X^baseform=sein
  & X.label=C3
  & X@cat=INF
  -> 
  subsumes( Sorten, X@args:1:sort, X^sort )
;

// Zustandspassiv: zweites Argument ist Subjekt zu sein
{X!SYN} : sein_main_c3_INFZU_sort : 0.5 :
  spec(X^id)
  & X^baseform=sein
  & X.label=C3
  & X@cat=INFZU
  & exists(X@args:2:sort) 
 -> 
  subsumes( Sorten, X@args:2:sort, X^sort )
;

// Wenn Zustandspassiv mit sein, dann haben Perfekt.
{X!SYN} : sein_main_c3_INFZU_haben : 0.5 :
  spec(X^id)
  & X^baseform=sein
  & X.label=C3
  & X@cat=INFZU
  -> 
  X@perfect=haben
;

// Perfekt-Form (auch fr haben!)
{X!SYN} : sein_aux_perfect : 0.2 :
  spec(X^id) 
  & X^cat=VAUXFIN
  & X@cat=PPP
  & X.label=C2
  ->
  X@perfect=X^baseform
;  

{X!SYN/\Y!SYN} : sein_aux_perfect_sorts : 0.3 :
  spec(X^id)
  & X^baseform=sein & X^cat=VAUXFIN
  & X.label=C1
  & Y@cat=PPP
  & Y.label=C2
  ->
  exists(Y@args:1:sort) 
  & subsumes(Sorten, Y@args:1:sort, X@sort) 
;
