# Copyright (C) 1997-2004 The CDG Team <cdg@nats.informatik.uni-hamburg.de>
#  
# This file is free software; as a special exception the author gives
# unlimited permission to copy and/or distribute it, with or without 
# modifications, as long as this notice is preserved.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
# implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

BLAH_VERSION=$(VERSION)
XCDG_VERSION=$(VERSION)
YADA_VERSION=2.0-alpha

DESTDIR=
CURDIR=$(shell pwd)
RELDIR=$(subst $(TOPDIR),.,$(CURDIR))
BINDINGSDIR=$(TOPDIR)/bindings
TESTGRAMMAR=$(TOPDIR)/grammar/stellingen/stellingen-flat.cdg
TESTSENTENCE=n001k000-1

BIN_DIR=$(PREFIX)/bin
LIB_DIR=$(PREFIX)/lib
MAN_DIR=$(PREFIX)/man/man1
DOCU_DIR=$(PREFIX)/share/doc/cdg
HEADER_DIR=$(PREFIX)/include/cdg
SHARE_DIR=$(PREFIX)/share/cdg
GRAMMAR_DIR=$(SHARE_DIR)/grammar

DEFAULT_INSTALL_DIRS=\
  $(PREFIX) \
  $(PREFIX)/lib \
  $(PREFIX)/include \
  $(PREFIX)/man \
  $(PREFIX)/share \
  $(PREFIX)/share/doc \
  $(BIN_DIR) \
  $(HEADER_DIR) \
  $(MAN_DIR) \
  $(SHARE_DIR) \
  $(DOCU_DIR) \
  $(GRAMMAR_DIR)

INSTALL_TARGETS=\
  pre-install \
  install-header \
  install-lib \
  install-bin \
  install-man \
  install-docu \
  install-files \
  install-cdgsetup \
  post-install

UNINSTALL_TARGETS=\
  pre-uninstall \
  uninstall-header \
  uninstall-lib \
  uninstall-bin \
  uninstall-man \
  uninstall-docu \
  uninstall-files \
  uninstall-cdgsetup \
  post-uninstall

# substitutions
SUBSTITUTIONS=\
  s,@VERSION\@,$(VERSION),g; \
  s,@BLAH_VERSION\@,$(BLAH_VERSION),g; \
  s,@XCDG_VERSION\@,$(XCDG_VERSION),g; \
  s,@YADA_VERSION\@,$(YADA_VERSION),g; \
  s,@PREFIX\@,$(PREFIX),g; \
  s,@TOPDIR\@,$(TOPDIR),g;\
  s,@CURDIR\@,$(CURDIR),g;\
  s,@LATEX\@,$(LATEX),g; \
  s,@MAKEINDEX\@,$(MAKEINDEX),g; \
  s,@BINDINGSDIR\@,$(BINDINGSDIR),g; \
  s,@PERL\@,$(PERL),g; \
  s,@TESTGRAMMAR\@,$(TESTGRAMMAR),g; \
  s,@TESTSENTENCE\@,$(TESTSENTENCE),g;

IGNORED_MAKE_OUTPUT='(Nothing to be done)|(is up to date)'

# programms
MAKE=make
CC=gcc
LD=ld
AR=ar
RANLIB=ranlib
LEX=flex
YACC=bison
LN=ln
SWIG=swig
GZIP=gzip
SED=sed
DOXYGEN=doxygen
POD2LATEX = $(TOPDIR)/utils/mypod2latex.pl

# programms sensitive to verbosity
ifdef VERBOSE
  TRUE=true
  ECHO=true
  FOR=for
  FIND=find
  IF=if
  CAT=cat
  CD=cd
else
  TRUE=@true
  ECHO=@echo
  FOR=@for
  FIND=@find
  IF=@if
  CAT=@cat
  CD=@cd
endif

# tags
ETAGS=etags
ETAGS_ARGS=--members
CTAGS=ctags
CTAGS_ARGS=--extra=+f --fields=fks 
TCL_ETAGS_ARGS = --lang=none \
  --regex='/proc[ \t]+\([^ \t]+\)/\1/' \
  --regex='/\(body\|configbody\)[ \t]+[^ \t]+::\([^ \t]+\)/\2/' \
  --regex='/class[ \t]+\([^ \t]+\)/\1/' \
  --regex='/[ \t]*\(private\|public\|protected\)[ \t]+variable[ \t]+\([^ \t]+\)/\2/'

# latex suite
PS2PDF_ARGS=-dMaxSubsetPct=100 \
  -dOptimize=true \
  -dUseFlateCompression=true \
  -dCompatibilityLevel=1.3 \
  -dSubsetFonts=true \
  -dEmbedAllFonts=true \
  -sPAPERSIZE=a4 \
  -dPDFSETTINGS=/printer
DVIPS_ARGS=-G0 -Pcmz -Pamz
#-Ppdf -G0 -Pcmz -Pamz
LATEX_ARGS=-interaction=nonstopmode

# for in-situ executables
RPATH=-Wl,-rpath -Wl,$(TOPDIR)/libcdg -Wl,-rpath -Wl,$(TOPDIR)/libcontainer

# swig
ifdef OLD_SWIG
  SWIG_TCL_FLAGS = -tcl8 -strict 0 -noobject -d tclcdg -DOLD_SWIG 
  SWIG_PERL_FLAGS = -perl5 -strict 0 -exportall -d libCDG -DOLD_SWIG
  SWIG_INTERFACE = cdg-old.i
else
  SWIG_TCL_FLAGS = -tcl8 -nodefault
  SWIG_PERL_FLAGS = -perl5 -exportall -nodefault 
  SWIG_INTERFACE = cdg.i
endif

# readline 
# TODO: inplement check_readline (reminder watch out for autoconfig.h)
READLINE_INCLUDE=
READLINE_LIB=-lreadline

# terminal
# TODO : where's tputs in (reminder watch out for autoconfig.h)
# optional: ncurses, termlib termcap curses
TERMINAL_LIB=-lncurses

# compiler flags
CFLAGS += -fPIC -fpic
#CFLAGS += -Wall -Werrror
# strong optimization
#CFLAGS += -O3 -ffast-math
# debug flags
#CFLAGS += -ggdb
# profile flags
#CFLAGS += -gprof ...

C_SOURCES = $(filter %.c,$(SOURCES))
H_SOURCES = $(filter %.h,$(SOURCES))
LEX_SOURCES = $(filter %.l,$(SOURCES))
YACC_SOURCES = $(filter %.y,$(SOURCES))

# deutsch lexicon converter
DEUTSCH_DIR = $(TOPDIR)/grammar/deutsch
MAKE_VERBS_PRG=$(DEUTSCH_DIR)/make-verbs.pl
MAKE_NAMES_PRG=$(DEUTSCH_DIR)/make-names.pl
MAKE_NOUNS_PRG=$(DEUTSCH_DIR)/make-nouns.pl
MAKE_ADJS_PRG=$(DEUTSCH_DIR)/make-adjectives.pl
MAKE_VERBS = $(PERL) -I$(TOPDIR)/include $(MAKE_VERBS_PRG)
MAKE_NAMES = $(PERL) -I$(TOPDIR)/include $(MAKE_NAMES_PRG)
MAKE_NOUNS = $(PERL) -I$(TOPDIR)/include $(MAKE_NOUNS_PRG)
MAKE_ADJS = $(PERL) -I$(TOPDIR)/include $(MAKE_ADJS_PRG)

.PHONY: $(SUBDIRS) clean distclean docu all check debug \
  install install-dir $(INSTALL_TARGETS) $(UNINSTALL_TARGETS) 
.SUFFIXES: .m4

# default rules
$(SUBDIRS):
	@$(MAKE) -C $@

status:
	$(ECHO) -e "\n" \
	  "VERSION=$(VERSION)\n" \
	  "BUILD=$(BUILD)\n" \
	  "BLAH_VERSION=$(BLAH_VERSION)\n" \
	  "XCDG_VERSION=$(XCDG_VERSION)\n" \
	  "YADA_VERSION=$(YADA_VERSION)\n\n" \
	  "TOPDIR=$(TOPDIR)\n" \
	  "CURDIR=$(CURDIR)\n" \
	  "RELDIR=$(RELDIR)\n" \
	  "PREFIX=$(PREFIX)\n" \
	  "DESTDIR=$(DESTDIR)\n" \
	  "BINDINGSDIR=$(BINDINGSDIR)\n\n" \
	  "BIN_DIR=$(BIN_DIR)\n" \
	  "LIB_DIR=$(LIB_DIR)\n" \
	  "MAN_DIR=$(MAN_DIR)\n" \
	  "DOCU_DIR=$(DOCU_DIR)\n" \
	  "HEADER_DIR=$(HEADER_DIR)\n" \
	  "SHARE_DIR=$(SHARE_DIR)\n" \
	  "GRAMMAR_DIR=$(GRAMMAR_DIR)\n" \
	  "DEFAULT_INSTALL_DIRS=$(DEFAULT_INSTALL_DIRS)\n\n" \
	  "NEGRADIR=$(NEGRADIR)\n" \
	  "WSJDIR=$(WSJDIR)\n" \
	  "CCCDIR=$(WSJDIR)\n\n" \
	  "PERL=$(PERL)\n" \
	  "PERL_CORE=$(PERL_CORE)\n" \
	  "PERL_INCLUDE=$(PERL_INCLUDE)\n" \
	  "PERL_FLAGS=$(PERL_FLAGS)\n" \
	  "PERL_LIBS=$(PERL_LIBS)\n\n" \
	  "TCL_INCLUDE=$(TCL_INCLUDE)\n" \
	  "TCL_LIBS=$(TCL_LIBS)\n\n" \
	  "CFLAGS=$(CFLAGS)\n" \
	  "LDFLAGS=$(LDFLAGS)\n" \
	  "LDLIBS=$(LDLIBS)\n" \
	  "INCLUDES=$(INCLUDES)\n" \
	  "RPATH=$(RPATH)\n\n" \
	  "BDB_INCLUDE=$(BDB_INCLUDE)\n" \
	  "BDB_LIB=$(BDB_LIB)\n\n" \
	  "READLINE_INCLUDE=$(READLINE_INCLUDE)\n" \
	  "READLINE_LIB=$(READLINE_LIB)\n\n" \
	  "TERMINAL_LIB=$(TERMINAL_LIB)\n\n" \
	  "SWIG_TCL_FLAGS=$(SWIG_TCL_FLAGS)\n" \
  	  "SWIG_PERL_FLAGS=$(SWIG_PERL_FLAGS)\n" \
  	  "SWIG_INTERFACE=$(SWIG_INTERFACE)\n\n"

tags: $(SOURCES)
	@if test -n "$(SOURCES)"; then \
	  echo "  CTAGS" && $(CTAGS) $(CTAGS_ARGS) $(SOURCES); \
	else \
	  touch tags; \
	fi; \
	for dir in $(SUBDIRS); do \
	  $(MAKE) -C $$dir tags || exit 1; \
	done

TAGS: $(SOURCES)
	@if test -n "$(SOURCES)"; then \
	  echo "  ETAGS" && $(ETAGS) $(ETAGS_ARGS) $(SOURCES); \
	else \
	  touch TAGS; \
	fi; \
	for dir in $(SUBDIRS); do \
	  $(MAKE) -C $$dir TAGS || exit 1; \
	done

dist-files: Makefile $(TOPDIR)/include/config.mk
	$(ECHO) "  GEN $@"; \
	rm -f dist-files; \
	dir=$(subst $(TOPDIR),cdg-$(VERSION),$(CURDIR)); \
	echo $$dir/Makefile >> dist-files; \
	if test -n "$(DIST_FILES)"; then \
	  for file in $(DIST_FILES); do \
	    echo $$dir/$$file >> $@; \
	  done \
	fi; \
	for dir in $(SUBDIRS); do \
	  $(MAKE) -C $$dir dist-files || exit 1; \
	  sort -u $$dir/dist-files >> $@; \
	done

clean::
	rm -f core *~ *.o tags TAGS *.stamp dist-files
	$(FOR) dir in $(SUBDIRS); do \
	  $(MAKE) -C $$dir clean || exit 1; \
	done

distclean:: clean
	rm -f deps cdgsetup.pl
	$(FOR) dir in $(SUBDIRS); do \
	  $(MAKE) -k -C $$dir distclean || exit 1; \
	done

check::
	$(FOR) dir in $(SUBDIRS); do \
	  $(MAKE) -C $$dir check || exit 1; \
	done

docu::
	$(FOR) dir in $(SUBDIRS); do \
	  $(MAKE) -C $$dir docu || exit 1; \
	done

cdgsetup.pl: $(TOPDIR)/include/config.mk $(TOPDIR)/include/autoconfig.mk 
	$(ECHO) "  GEN $@" && \
	( \
	  echo "# CAUTION: This file has been generated automatically !!!"; \
	  echo "# CDG version $(VERSION), build $(BUILD)"; \
	  echo "use lib \"$(BINDINGSDIR)\"; 1;"; \
	) > $@

$(TOPDIR)/libcontainer/libblah.so:
	$(MAKE) -C $(TOPDIR)/libcontainer libblah.so

$(TOPDIR)/libcdg/libcdg.so:
	$(MAKE) -C $(TOPDIR)/libcdg libcdg.so

$(TOPDIR)/bindings/libCDG.so:
	$(MAKE) -C $(TOPDIR)/bindings libCDG.so

$(TOPDIR)/bindings/tclcdg.so:
	$(MAKE) -C $(TOPDIR)/bindings tclcdg.so


# installation suite
install:: $(INSTALL_TARGETS)

pre-install::
	$(FOR) file in $(RELINK_FILES); do \
	  rm -f $$file; \
	  $(MAKE) --no-print-directory RPATH?="-Wl,-rpath -Wl,$(LIB_DIR)" $$file || exit 1; \
	done

post-install::
	$(FOR) dir in $(SUBDIRS); do \
	  $(MAKE) -C $$dir install || exit 1; \
	done; \
	for file in $(RELINK_FILES); do \
	  rm -f $$file; \
	  $(MAKE) --no-print-directory $$file || exit 1; \
	done

install-dir::
	$(IF) test -n "$(DESTDIR)"; then \
	  if test  ! -d $(DESTDIR); then \
	    echo "  MKDIR $(DESTDIR)"; mkdir -p -m 755 $(DESTDIR); \
	  fi \
	fi
	$(FOR) dir in $(DEFAULT_INSTALL_DIRS) $(INSTALL_DIRS) $(INSTALL_DIR); do \
	  target=$(DESTDIR)$$dir; \
	  if test ! -d $$target; then \
	    echo "  MKDIR $$target"; \
	    mkdir -p -m 755 $$target; \
	  fi \
	done

install-files:: install-dir
	$(IF) test -n "$(INSTALL_FILES)"; then \
	  $(MAKE) --no-print-directory $(INSTALL_FILES) | \
	  egrep -v $(IGNORED_MAKE_OUTPUT); \
	  for file in $(INSTALL_FILES); do \
	    target=$(DESTDIR)$(INSTALL_DIR)/$$file; \
	    echo "  INSTALL $$target"; \
	    $(INSTALL) -m 644 $$file $$target; \
	  done \
	fi

install-header:: install-dir
	$(IF) test -n "$(INSTALL_HEADERS)"; then \
	  for file in $(INSTALL_HEADERS); do \
	    target=$(DESTDIR)$(HEADER_DIR)/$$file; \
	    echo "  INSTALL $$target"; \
	    $(INSTALL) -m 644 $$file $$target; \
	  done \
	fi

install-lib:: install-dir
	$(IF) test -n "$(INSTALL_LIBS)"; then \
	  $(MAKE) --no-print-directory $(INSTALL_LIBS) | \
	  egrep -v $(IGNORED_MAKE_OUTPUT); \
	  for file in $(INSTALL_LIBS); do \
	    target=$(DESTDIR)$(LIB_DIR)/$$file; \
	    echo "  INSTALL $$target"; \
	    $(INSTALL) -m 644 $$file $$target; \
	  done \
	fi

install-bin:: install-dir 
	$(IF) test -n "$(INSTALL_PROGS)"; then \
	  $(MAKE) --no-print-directory $(INSTALL_PROGS) | \
	  egrep -v $(IGNORED_MAKE_OUTPUT); \
	  for file in $(INSTALL_PROGS); do \
	    target=$(DESTDIR)$(BIN_DIR)/$$file; \
	    echo "  INSTALL $$target"; \
	    $(INSTALL) -m 755 $$file $$target; \
	  done \
	fi

install-man:: install-dir
	$(IF) test -n "$(INSTALL_MANS)"; then \
	  $(MAKE) --no-print-directory $(INSTALL_MANS) | \
	  egrep -v $(IGNORED_MAKE_OUTPUT); \
	  for file in $(INSTALL_MANS); do \
	    target=$(DESTDIR)$(MAN_DIR)/$$file; \
	    echo "  INSTALL $$target"; \
	    $(INSTALL) -m 644 $$file $$target; \
	  done \
	fi

install-docu:: install-dir
	$(IF) test -n "$(INSTALL_DOCU)"; then \
	  $(MAKE) --no-print-directory $(INSTALL_DOCU) | \
	  egrep -v $(IGNORED_MAKE_OUTPUT); \
	  for file in $(INSTALL_DOCU); do \
	    target=$(DESTDIR)$(DOCU_DIR)/$$file; \
	    echo "  INSTALL $$target"; \
	    $(INSTALL) -m 644 $$file $$target; \
	  done \
	fi

install-cdgsetup:: install-dir
	$(IF) test -n "$(INSTALL_CDGSETUP)"; then \
	  target=$(DESTDIR)$(INSTALL_CDGSETUP); \
	  echo "  LN $$target"; \
	  ln -sf $(LIB_DIR)/cdg/cdgsetup.pl $$target; \
	fi

uninstall: $(UNINSTALL_TARGETS)

pre-uninstall:: 
	$(FOR) dir in $(SUBDIRS); do \
	  $(MAKE) -C $$dir uninstall || exit 1; \
	done;

uninstall-bin::
	$(FOR) file in $(INSTALL_PROGS); do \
	  target=$(DESTDIR)$(BIN_DIR)/$$file; \
	  if test -f $$target; then \
	    echo "  RM $$target"; \
	    rm $$target; \
	  fi \
	done;

uninstall-lib::
	$(FOR) file in $(INSTALL_LIBS); do \
	  target=$(DESTDIR)$(LIB_DIR)/$$file; \
	  if test -f $$target; then \
	    echo "  RM $$target"; \
	    rm $$target; \
	  fi \
	done;

uninstall-header::
	$(FOR) file in $(INSTALL_HEADERS); do \
	  target=$(DESTDIR)$(HEADER_DIR)/$$file; \
	  if test -f $$target; then \
	    echo "  RM $$target"; \
	    rm $$target; \
	  fi \
	done;

uninstall-man::
	$(FOR) file in $(INSTALL_MANS); do \
	  target=$(DESTDIR)$(MAN_DIR)/$$file; \
	  if test -f $$target; then \
	    echo "  RM $$target"; \
	    rm $$target; \
	  fi \
	done;

uninstall-docu::
	$(FOR) file in $(INSTALL_DOCU); do \
	  target=$(DESTDIR)$(DOCU_DIR)/$$file; \
	  if test -f $$target; then \
	    echo "  RM $$target"; \
	    rm $$target; \
	  fi \
	done;
	
uninstall-files:: 
	$(FOR) file in $(INSTALL_FILES); do \
	  target=$(DESTDIR)$(INSTALL_DIR)/$$file; \
	  if test -f $$target; then \
	    echo "  RM $$target"; \
	    rm $$target; \
	  fi \
	done

uninstall-cdgsetup::
	$(IF) test -n "$(INSTALL_CDGSETUP)";then \
	  target=$(DESTDIR)$(INSTALL_CDGSETUP); \
	  if test -h "$$target"; then \
	    echo "  RM $$target"; rm -f $$target; \
	  fi \
	fi

post-uninstall::
	$(TRUE); dirs=`for dir in $(DEFAULT_INSTALL_DIRS) $(INSTALL_DIRS) $(INSTALL_DIR); do echo $$dir; done | tac`; \
	for dir in $$dirs; do \
	  target=$(DESTDIR)$$dir; \
	  if test -d $$target; then \
	    if test -z "`ls -1 $$target`"; then \
	      echo "  RMDIR $$target " && rmdir $$target; \
	    fi; \
	  fi; \
	done; \
	if test -n "$(DESTDIR)"; then \
	  if test -d $(DESTDIR); then \
	    if test -z "`ls -1 $(DESTDIR)`"; then \
	      echo "  RMDIR $(DESTDIR) " && rmdir $(DESTDIR); \
	    fi; \
	  fi; \
	fi
	  
# pattern rules
.SUFFIXES: .o .c .l .m4 .y .so .a .d .1 .pl .pm .gz .cdg \
	   .tex .dvi .ps .gz .pdf .bbl .bib .in .sh .stamp .tcl

%.o: %.c
	$(ECHO) "  CC $<" && $(CC) -c $(CFLAGS) $(INCLUDES) $< -o $@

%: %.o
	$(ECHO) "  LD $@" && $(CC) $(LDFLAGS) $(RPATH) $< $(LDLIBS) -o $@

%.l: %.l.m4
	$(ECHO) "  M4 $@" && $(M4) $< > $@


%.c : %.l
	$(ECHO) "  LEX $@" && $(LEX) -t $< > $@

%.c: %.y
	$(ECHO) -n "  YACC $@ " && $(YACC) $< -d -o $@

%.so: %-$(VERSION).so
	$(ECHO) "  LN $@" && $(LN) -sf $< $@

%.a: %-$(VERSION).a
	$(ECHO) "  LN $@" && $(LN) -sf $< $@

%.d: %.c
	$(ECHO) "  DEPS $@" && $(CC) -MM $(INCLUDES) $^ > $@

%.1: %.pl
	$(ECHO) "  POD $@" && $(POD2MAN) $< $@

%.1: %.pm
	$(ECHO) "  POD $@" && $(POD2MAN) $< $@

%.cdg: %.m4
	$(ECHO) "  M4 $@" && $(M4) -s -P -I. $< > $@

%.bbl: %.bib
	$(ECHO) "  LATEX $@" && $(LATEX) $(LATEX_ARGS) $*
	$(ECHO) "  BIBTEX $@" && $(BIBTEX) $*

%.ps: %.dvi
	$(ECHO) "  DVIPS $@" && $(DVIPS) $(DVIPS_ARGS) $< -o $@ 2>/dev/null

%.pdf: %.ps
	$(ECHO) "  PS2PDF $@" && $(PS2PDF) $(PS2PDF_ARGS) $< $@

%.dvi: %.tex
	$(TRUE); \
	( \
	  latex_count=0; \
	  while [ $$latex_count -le 1 ]; do \
	    echo "  LATEX  [$$latex_count] $@"; \
	    $(LATEX) $(LATEX_ARGS) $* 1>&2 || exit 1; \
	    latex_count=`expr $$latex_count + 1`; \
	  done; \
	  while grep "LaTeX Warning: Citation" $*.log > /dev/null 2>&1 && \
	    [ $$latex_count -le 3 ]; do \
	    ( \
	      echo "  BIBTEX [$$latex_count] $*" && $(BIBTEX) $* 1>&2 && \
	      echo "  LATEX  [$$latex_count] $@" && $(LATEX) $(LATEX_ARGS) $* 1>&2; \
	    ) || exit 1; \
	    latex_count=`expr $$latex_count + 1`; \
	  done; \
	  test -z "`grep "LaTeX Warning: Citation" $*.log`" && \
	  while grep "LaTeX Warning: Label(s) may have changed." $*.log > /dev/null 2>&1 && \
	    [ $$latex_count -le 6 ]; do \
	    echo "  LATEX  [$$latex_count] $@" && $(LATEX) $(LATEX_ARGS) $* 1>&2 || exit 1; \
	    latex_count=`expr $$latex_count + 1`; \
	  done; \
	  test -z "`grep "LaTeX Warning: Label(s) may have changed." $*.log`" \
	) 2> $*.stderr || \
	( \
	  cat $*.stderr; \
	  echo "ERROR: ... see above ($*.stderr)"; \
	  rm -f $*.aux $*.toc $*.log $*.dvi $*.bbl $*.blg; \
	  exit 1; \
	)

%.gz: %
	$(ECHO) "  GZIP $@" && $(GZIP) -fc $< > $@

%: %.in
	$(ECHO) "  SED $@" && $(SED) '$(SUBSTITUTIONS)' $< >$@

%: %.sh
	$(ECHO) "  SED $@" && $(SED) '$(SUBSTITUTIONS)' $< >$@ && chmod a+x $@

%.pl: %.pl.sh
	$(ECHO) "  SED $@" && $(SED) '$(SUBSTITUTIONS)' $< >$@ && chmod a+x $@

%: %.pl
	$(ECHO) "  SED $@" && $(SED) '$(SUBSTITUTIONS)' $< >$@ && chmod a+x $@

%.tcl: %.tcl.sh
	$(ECHO) "  SED $@" && $(SED) '$(SUBSTITUTIONS)' $< >$@ && chmod a+x $@

%: %.tcl
	$(ECHO) "  SED $@" && $(SED) '$(SUBSTITUTIONS)' $< >$@ && chmod a+x $@


%.stamp:
	$(ECHO) "  STAMP $*" && touch $@

