/* Copyright (C) 1997-2004 The CDG Team <cdg@nats.informatik.uni-hamburg.de>
 *  
 * This file is free software; as a special exception the author gives
 * unlimited permission to copy and/or distribute it, with or without 
 * modifications, as long as this notice is preserved.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

/* -------------------------------------------------------------------------
 * @addtogroup Chunker 
 * 
 * $Id: chunker.h,v 1.12 2004/09/01 13:44:12 micha Exp $
 *
 * @{
 */


#ifndef CHUNKER_H
#define CHUNKER_H

/* -- INCLUDES ------------------------------------------------------------- */
#include <cdg.h>
#include <lexemgraph.h>

/* -- TYPE DEFINITIONS ----------------------------------------------------- */

/* ------------------------------------------------------------------------- 
 * different modes the chunker can operate in.
 */
typedef enum {
  DefaultChunker,	     /**< the globally set chunker type: one of the below */
  FakeChunker,		     /**< read chunks from the annotation */
  RealChunker,		     /**< call the real chunker */
  EvalChunker,		     /**< call the real chunker and compare it to the fake chunker */
} ChunkerMode;

/* ------------------------------------------------------------------------- 
 * chunk types.
 */
typedef enum {
  NChunk,		     /**< a chunk of a nominal clause */
  PChunk,		     /**< a chunk of a prepositional clause */
  VChunk,		     /**< a chunk of a verbal clause */
  NoChunk,		     /**< things which never go into a chunk */
  UnknownChunk		     /**< fallback */
} ChunkType;

/* ------------------------------------------------------------------------- 
 * internal representation of a chunk.
 */
struct ChunkStruct {
  ChunkType type;	     /**< label of the chunk */
  List nodes;		     /**< list of all lexem nodes in the chunk */
  GraphemNode from;	     /**< first element in the chunk */
  GraphemNode to;	     /**< last element in the chunk */
  GraphemNode head;	     /**< head of the chunk */
  struct ChunkStruct *parent;	/**< direct dominating span (only used in fake-chunking */
  List subChunks;	     /**< embedded chunks, e.g. [PC ... [NC ...]] */
};


/* -- FUNCTIONS ------------------------------------------------------------ */
 
/* initialize chunker module */
extern void chunkerInitialize(void);

/* finalize the chunker module */
extern void chunkerFinalize(void);

/* construct a new chunker object */
extern Chunker chunkerNew(ChunkerMode mode, LexemGraph lg);

/* destroy a chunker object */
extern void chunkerDelete(Chunker chunker);

/* destroy a chunk object */
extern void chunkerChunkDelete(Chunk chunk);

/* compute the chunks for a lexemgraph */
extern List chunkerChunk(Chunker chunker);

/* pretty print the given chunks */
void chunkerPrintChunks(unsigned long mode, List chunks);

/* get the string representation of a chunk type */
extern String chunkerStringOfChunkType(Chunk chunk);

/* get the chunk type of its string representation */
extern ChunkType chunkerChunkTypeOfString(String tag);

/* chunkerCloneChunk: construct a copy of a given chunk including clones of subChunks. */
extern Chunk chunkerCloneChunk(Chunk chunk);

/* replace all grapheme references in a chunk with those given in a lexemgraph */
void chunkerReplaceGraphemes(Chunk chunk, LexemGraph lg);

/* ------------------------------------------------------------------------- */
#endif /* don't insert anything after this #endif */

