/* 
 * Copyright (C) 1997-2004 The CDG Team <cdg@nats.informatik.uni-hamburg.de>
 *  
 * This file is free software; as a special exception the author gives
 * unlimited permission to copy and/or distribute it, with or without 
 * modifications, as long as this notice is preserved.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * Author: Ingo Schroeder
 * Birth: 6/5/97
 *
 * $Id: netsearch.h,v 1.33 2004/06/09 12:38:15 foth Exp $
 */

#ifndef NETSEARCH_H
#define NETSEARCH_H

/* -------------------------------------------------------------------------
 * @addtogroup Netsearch
 * @{ 
 */

/* ---------------------------------------------------------------------- */
#include <blah.h>
#include "input.h"
#include "lexemgraph.h"
#include "constraintnet.h"
#include "levelvalue.h"
#include "eval.h"

/* ---------------------------------------------------------------------- */
typedef struct NetSearchStateStruct {
  int id;		     /**< identifies a search state within one search */
  ConstraintNet net;	     /**< constraint net we search in */
  Badness badness;	     /**< cumulative badness of this state */
  int nextNodeIndex;	     /**< index of node to be bound next */
  ByteVector isDeletedLexemNode;	/**< booleans indicating lexem usage in this state */
  NodeBinding nodeBinding;   /**< nodebinding at this searchstate */
  struct NetSearchStateStruct *parent;	/**< state which spawned us */
  int noOfUndeletedChildren; /**< number of children that are still valid */
  Boolean isDeletedState;    /**< temporary flag indicating an invalid nodebinding */
  int noOfBindings;	     /**< number of bindings we made at this state */
  int noOfDeletedBindings;   /**< number of skiped bindings */
} NetSearchStateStruct;
typedef NetSearchStateStruct *NetSearchState;

#ifndef SWIG
typedef List ListFunction (); 
typedef struct {
   String id;			       /**< identifying name */
   ListFunction *doIt;	       /**< function to search */
   String doc;			       /**< short help */
} SearchMethod;
#endif

typedef enum {
  NSNOff,
  NSNLinear,
  NSNSquare,
  NSNDepth,
  NSNBreadth,
} NsNormalizationType;

typedef enum {
   STBranchBound,
   STFullSearch
} SearchType;


/* ---------------------------------------------------------------------- */

#ifndef SWIG
extern SearchMethod search_methods[];

/**
 * if TRUE, netsearch tries to find a matching annotation
 * and displays recall measures for intermediate solutions.
 */
extern Boolean nsAutoCompare;

#endif

/**
 * If TRUE, a successful search also modifies the underlying net,
 * i. e. the levelvalues that aren't part of a solution are deleted. 
 */
extern Boolean nsSearchModifiesNetFlag;


/**
  type of agenda normalization. 
  - off:     wie oben
  - linear:  wie on oben
  - square:  agenda score == n-te Wurzel aus search state score
             mit n gleich no of undeleted bindings
  - depth:   agenda score monoton steigend (=> depth-first search)
  - breadth: agenda score == no of undeleted bindings
*/
extern NsNormalizationType nsNormalization;

/* ---------------------------------------------------------------------- */

#ifndef SWIG
extern int netsearch(ConstraintNet net, int no, char **args);
extern void nsDeleteNetSearchState(NetSearchState nss);
extern Badness nsCompatibleBindings(NetSearchState nss, LevelValue lv, ByteVector isDeletedLexemNode);
extern void nsSkipOverDeletedNodes(NetSearchState nss);
extern void nsSkipOverDeletedAndUsedNodes(NetSearchState nss);
extern Boolean nsIsValidSolution(NetSearchState nss);
extern Number nsComputeAgendaScore(NetSearchState nss);
extern int nsPrint(unsigned long mode, NetSearchState nss, int no);
extern int nsComputeOverlayedStates(NetSearchState nss);
extern Boolean nsLexemesValid(List lexemes, NetSearchState nss);
extern List nsBranchBound(ConstraintNet net, int no, char **args);
extern void nsRegisterParses(ConstraintNet net, List parses);
extern void nsInitialize();
extern List nsSearch(ConstraintNet net, SearchType mode, int agendasize, Number threshold);
#endif

/* ---------------------------------------------------------------------- */
/** @} */
#endif /* don't insert anything after this #endif */
