/* 
 * Copyright (C) 1997-2004 The CDG Team <cdg@nats.informatik.uni-hamburg.de>
 *  
 * This file is free software; as a special exception the author gives
 * unlimited permission to copy and/or distribute it, with or without 
 * modifications, as long as this notice is preserved.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * Author: Horia F. Pop
 * Purpose: demo for libcdg/vars.c
 * Date: February 11, 2000
 *
 * $Id: settest.c,v 1.1 2004/09/22 15:53:55 micha Exp $ 
 */

#include <assert.h>
#include <stdio.h>
#include "config.h"
#include "cdg.h"
#include "hook.h"
#include "set.h"

//#define VERBOSE

Boolean setFunctionInt(String name, String value, int *var, int *cnv)
{
  int val;

  if (sscanf(value, "%d", &val)) {
    switch (val) {
      case 10:
      case 20:
      case 30:
      case 50:
	*cnv = val;
	return TRUE;
      default:
	return FALSE;
    }
  } else {
    return FALSE;
  }
}

void setDemo(void)
{
  char sc1[] = "val-sc1";
  String s1 = NULL;
  int i1 = 10;
  float f1 = 1.5;
  Boolean bi, bs, bf;

  void setPrintVars(String s) {
#ifdef VERBOSE
    cdgPrintf(CDG_INFO, "%s\n", s);
    setPrint(CDG_INFO, "i1");
    setPrint(CDG_INFO, "f1");
    setPrint(CDG_INFO, "s1");
    cdgPrintf(CDG_INFO, "\n");
#endif
  }

  void setPrintBoolean(String s, Boolean b) {
#ifdef VERBOSE
    cdgPrintf(CDG_INFO, "%s ", s);
    if (b) {
      cdgPrintf(CDG_INFO, "success\n");
    } else {
      cdgPrintf(CDG_INFO, "failure\n");
    }
#endif
    assert(b);
  }


  s1 = sc1;

  bi =
    setRegister("i1", SET_INT, &i1, (BooleanFunction *) setFunctionInt, NULL,
		NULL, NULL, NULL);
  bf = setRegister("f1", SET_FLOAT, &f1, NULL, NULL, NULL, NULL);
  bs = setRegister("s1", SET_STRING, &s1, NULL, NULL, NULL, NULL);

  setPrintVars("Initial");

  setPrintBoolean("setRegister on i1 returned", bi);
  setPrintBoolean("setRegister on f1 returned", bf);
  setPrintBoolean("setRegister on s1 returned", bs);

  setPrintVars("After registering");

  bi = setValue("i1", "50");
  bf = setValue("f1", "2.5");
  bs = setValue("s1", strRegister("long-string"));

  /* possible calls with string argument: varSet("s1", sc3); varSet("s1",
   * s3); varSet("s1", strRegister("long-string")); varSet("s1", "foo"); */

  setPrintBoolean("setValue on i1 returned", bi);
  setPrintBoolean("setValue on f1 returned", bf);
  setPrintBoolean("setValue on s1 returned", bs);

  setPrintVars("After set");

  s1 = strRegister("val-modif");
  i1 = 30;
  f1 = 3.5;

  setPrintVars("After local change");
}

int main(void)
{
  cdgInitialize();
  setDemo();
  cdgFinalize();
  return 0;
}
