#!/usr/bin/env perl
# cdg/utils/callgraph.pl

use warnings;
use strict;
use Getopt::Std;

our($comment, $current,
    %path, %functions, %distance, %calls, %is_called_by);

# get args
our(
    $opt_a,     # report All functions: even those from other files
    $opt_r,     # print Reverse call relationship
    $opt_t      # print Transitive call relationship
   );
getopts('art');


$current="";
while (<>) {

  # ignore comments
  if (/\/\*/) {
    $comment = 1;
  }

  # ignore strings
  s/\"(\\\"|[^\"])+\"/CENSORED/g;

  if (!$comment) {
    if (/^([a-zA-Z0-9_]+\s+)+([a-zA-Z0-9_]+)\s*\(/) {
      $current = $2;
      $functions{$current}++;
    } elsif ($current) {
      while (s/([0-9a-zA-Z_]+)\s*\(//) {
        my $callee = $1;
        next if $callee =~ /^(do|for|if|return|switch|while)$/;
        $calls{$current}{$callee}++;
        $is_called_by{$callee}{$current}++;
      }
    }
  }

  if (/\*\//) {
    $comment = 0;
  }

}

for my $caller (keys %calls) {
  for (keys %{$calls{$caller}}) {
    $distance{$caller}{$_} = 1;
  }
}

for my $caller (keys %functions) {
  for my $callee (keys %functions) {
    $path{$caller}{$callee} = "";
  }
}

if (defined($opt_t)) {

  my $progress = 0;
  do {
    $progress = 0;
    for my $caller (keys %functions) {
#      print "caller $caller\n";
      for my $middleman (keys %{$distance{$caller}}) {
#        print "  middleman $middleman\n";
        my $a = $distance{$caller}{$middleman};
        for my $callee (keys %{$distance{$middleman}}) {
          next unless defined($opt_a) or exists($functions{$callee});
          my $b = $distance{$middleman}{$callee};
          if (defined($b)) {
#            print "    callee $callee\n";
            my $c = $distance{$caller}{$callee};
            $c = 99999 unless defined($c);
            if ($c > $a + $b) {
#              print "\ndistance $caller-$middleman == $a, distance $middleman-$callee == $b\n  ==>";
              $distance{$caller}{$callee} = $a + $b;
              $path{$caller}{$callee} =
                $path{$caller}{$middleman} . "$middleman:" . $path{$middleman}{$callee};
              $progress++;
            }
          }
        }
      }
    }
  } while $progress;
}

if (defined($opt_r)) {
  for my $f (sort keys %functions) {
    for (sort keys %{$is_called_by{$f}}) {
      print "$f() is called by $_()\n"
        if defined($opt_a) or exists $functions{$_};
    }
  }
} else {
  for my $caller (sort keys %functions) {
    for my $callee (sort keys %{$distance{$caller}}) {
      if(defined($opt_a) or exists $functions{$caller}) {
        print "$caller() calls $callee() ";
        my $x = $distance{$caller}{$callee};
        my $y = $path{$caller}{$callee};
        print $x == 1 ? "\n" : "($x hops: $caller:$y$callee)\n";
      }
    }
  }
}


for (sort keys %functions) {
  print "$_ is never called\n" unless keys %{$is_called_by{$_}};
}

