#!/usr/bin/env perl
# cdg/utils/cdg2png.pl

use warnings;
use strict;
use GD;

our $str =
  '(?:\'(?:(?:[^\\\\\']|\\\\.)*)\'|' .
  '"(?:(?:[^\\\\"]|\\\\.)*)"|' .
  '[a-zA-Z_\x80-\xff][a-zA-Z_0-9\x80-\xff]*)';

sub parse_string {
  my $s = shift;
  if ($s =~ /^([\'\"])(.*)\1$/) {
    $s = $2;
    $s =~ s|\\||g;
  }
  return $s;
}


our $sf=gdGiantFont;
our $im;
our $h;

sub String {
  my($f, $x, $y, $s, $c)=@_;
  $im->string($f, $x-int((length($s)*$f->width)/2), $h-$y-$f->height, $s, $c);
}

sub Circle {
  my($x, $y, $c)=@_;
  $im->arc($x, $h-$y,10,10,0,360,$c);
  $im->fill($x,$h-$y,$c);
}

sub Line {
  my($x1, $y1, $x2, $y2, $c)=@_;
  $im->line($x1, $h-$y1, $x2, $h-$y2, $c);
}

our @words;
our @edges;
our @labels;
our @children;
our @x;
our @level;

sub y_coord {
  my $n = shift;
  40 + 100*$level[$n];
}

sub height {
  my $x = shift;
  my $result = 0;
  while($x) {
    $x = $edges[$x];
    $result++;
  }
  return $result;
}


sub siblingCheck {
  my $down = shift;
  my $up = $edges[$down];
  my @l = $down<$up ? ($down..$up-1) : ($up-1..$down);
  for(@l) {
    return 0 if $edges[$_] != $up;
  }
  return 1;
}

# read edge declarations
my $cursor = 0;
while(<>) {

  # match "0 1 'nur' SYN -> 'ADV' -> 3"
  if(my($fr,$to,$word,$label,$gov) =
     /^\s*(\d+)\s+(\d+)\s+($str)\s+$str\s*->\s*($str)\s*->\s*(\d+)/) {
#    print STDERR "$to $word $gov\n";
    $words[$to] = parse_string $word;
    $edges[$to] = $gov;
    $labels[$to] = parse_string $label;
    $cursor += 10*length($words[$to]);
    $x[$to] = $cursor;
    $cursor += 10*length($words[$to]);
    push @{$children[$gov]}, $to;
  }
}


# compute vertical positions
my $max = 0;
for(1..$#edges) {
  my $y = height($_);
  $max = $y if $max < $y;
#  print STDERR "`$words[$_]': level $y\n";
  $level[$_] = $y;
}

for(1..$#edges) {
  # isolated node
  if(!$children[$_] && !$edges[$_]) {
    # lower rim
    $level[$_] = 0
  }

  # rot node
  elsif(!$edges[$_]) {
    # upper rim
    $level[$_] = $max
  }

  # leaf node
  elsif(!$children[$_]) {
    if(siblingCheck($_)) {
      $level[$_] = 0;
    } else {
      $level[$_] = $max - $level[$_];
    }
  }

  # inner node
  else {
    $level[$_] = $max - $level[$_];
  }
}
$level[0] = 0;


# create graphical items
$h = 100 + 100 * $max;
$im=new GD::Image($cursor + 20, $h);
our $black=$im->colorAllocate(0, 0, 0);
our $white=$im->colorAllocate(255, 255, 255);
$im->fill(1, 1, $white);

for(1..$#edges) {
  String($sf,$x[$_],10,$words[$_],$black);
  Circle($x[$_],y_coord($_),$black);
}
for(1..$#edges) {
  my $g = $edges[$_];
  my $x1 = $x[$_];
  my $y1 = y_coord($_);
  my $x2 = $edges[$_] ? $x[$edges[$_]] : $x[$_];
  my $y2 = y_coord($edges[$_]);

  # root edge
  if($y2 < $y1) {
    $y2 = $y1 + 40;
  }

  Line($x1,$y1, $x2,$y2, $black);
  String($sf, 10 + $x1+0.5*($x2-$x1), $y1+0.5*($y2-$y1), $labels[$_], $black);
}


print $im->png;
