#!/usr/bin/env perl
# cdg/utils/extract-grammar.pl
#
# This program tries to deduce constraints from a corpus of annotated
# sentences by computing the number of times that certain features
# (label, direction, category etc.) coincide. It is extremely
# incomplete yet.
use warnings;
use strict;
BEGIN {
  use File::Basename;
  unshift @INC, dirname(readlink($0) || $0); 
  require 'cdgsetup.pl';
}
use CDG;
use CDG::Utils qw( :functions);


sub distance {
  my $e = shift;
  if (0 == $$e{'^'}{no}) {
    return 0
  } else {
    return $$e{'@'}{no} - $$e{'^'}{no};
  }
}



commandEval "set verbosity 0 ";
commandEval "load @ARGV";
our %annotations = import_annos();

our %stat;
our %labels;
our %cats;

# stats:
# 
# directions -> labels x dir 
# distance   -> label x dist 
# proj1-6    -> label x label 
# punc       -> label x {0,1} 
# order	     -> label:label x {pre, post}
# botcat     -> cat x label
# topcat     -> cat x label
# cooccur    -> {/\, /} x label:label
# has        -> cat x label

for my $a (values %annotations) {

  for my $e (@{$$a{edges}}) {
    next unless $$e{level} eq 'SYN';

    # direction
    my $label = $$e{label};
    $labels{$label}++;
    my $dist = distance $e;
    my $dir = $dist ? int($dist / abs($dist)) : 0;
    $stat{directions}{$label}{$dir}++;
    $stat{distances}{$label}{$dist}++;

    # projectivity
    if ($dist) {
      my $x = $e;
      my $dx = $dist;
      for my $y (@{$$a{edges}}) {
        next unless $$y{level} eq 'SYN';
        my $dy = distance $y;
        next unless $dy;

        if ($$x{'^'}{no} == $$y{'@'}{no}) {

          # type 1
          if ($dx < 0 && $dy > 0 && $$y{'^'}{no} > $$x{'@'}{no}) {
            $stat{proj1}{"$$x{label}:$$y{label}"}++;
          }

          # type 2
          elsif($dx > 0 && $dy < 0 && $$y{'^'}{no} < $$x{'@'}{no}) {
            $stat{proj2}{"$$x{label}:$$y{label}"}++;
          }

        }

        # type 3
        elsif ($dx > 0 && $dy > 0 &&
               $$y{'^'}{no} < $$x{'@'}{no} &&
               $$y{'^'}{no} > $$x{'^'}{no} &&
               $$y{'@'}{no} > $$x{'@'}{no}) {
          $stat{proj3}{"$$x{label}:$$y{label}"}++;
        }

        # type 4
        elsif ($dx > 0 && $dy < 0 &&
               $$y{'@'}{no} > $$x{'^'}{no} &&
               $$y{'@'}{no} < $$x{'@'}{no} &&
               $$y{'^'}{no} > $$x{'@'}{no}) {
          $stat{proj4}{"$$x{label}:$$y{label}"}++;
        }

        # type 5
        elsif ($dx < 0 && $dy > 0 &&
               $$y{'@'}{no} > $$x{'^'}{no} &&
               $$y{'^'}{no} < $$x{'^'}{no} &&
               $$y{'^'}{no} > $$x{'@'}{no}) {
          $stat{proj5}{"$$x{label}:$$y{label}"}++;
        }

        # type 6
        elsif ($dx < 0 && $dy < 0 &&
               $$y{'^'}{no} > $$x{'^'}{no} &&
               $$y{'@'}{no} < $$x{'^'}{no} &&
               $$y{'@'}{no} > $$x{'@'}{no}) {
          $stat{proj6}{"$$x{label}:$$y{label}"}++;
        }
      }
    }

    # punctuation
    if($dist) {
      my $punc = 0;
      my @words = @{$$a{words}};
      if($dist < 0) {
        for($$e{'@'}{no}+1..$$e{'^'}{no}-1) {
          if($words[$_]{word} =~ /^[.,;()-]/) {
            $punc = 1;
          }
        }
      } else {
        for($$e{'^'}{no}+1..$$e{'@'}{no}-1) {
          if($words[$_]{word} =~ /^[.,;()-]/) {
            $punc = 1;
          }
        }
      }
      $stat{punc}{$label}{$punc}++;
    }

    # order
    for my $f (@{$$a{edges}}) {
      next if $f == $e;
      next unless $$f{level} eq 'SYN';
      next unless $$f{'^'}{no};
      next unless $$f{'^'}{no} == $$e{'^'}{no};
      if($$e{'@'}{no} < $$f{'@'}{no}) {
        $stat{order}{"$$e{label}:$$f{label}"}{pre}++;
      } else {
        $stat{order}{"$$e{label}:$$f{label}"}{post}++;
      }
    }

    # categories per label
    $cats{$$e{'@'}{cat}}++;
    $stat{botcat}{$$e{'@'}{cat}}{$label}++;
    if($$e{'^'}{no}) {
      $stat{topcat}{$label}{$$e{'^'}{cat}}++;
    }

    # label co-occurrence
    for my $f (@{$$a{edges}}) {
      next if $f == $e;
      next unless $$f{level} eq 'SYN';
      next unless $$f{'^'}{no};
      if($$f{'^'}{no} == $$e{'^'}{no}) {
        $stat{cooccur}{'/\\'}{"$$e{label}:$$f{label}"}++;
      }
      elsif($$f{'^'}{no} == $$e{'@'}{no}) {
        $stat{cooccur}{'/'}{"$$e{label}:$$f{label}"}++;
        $stat{has}{$$e{'@'}{cat}}{$$f{label}}++;
      }
    }
  }
}

## Create output

# level declaration
print "SYN # ";
my $no = 0;
for(sort keys %labels) {
  print ", " if $no++;
  print format_string $_;
}
print ";\n\n";

print "{X:REF} : _REF : auto : 0.0 : X|;\n\n";

# Closure
#
# If the training corpus does not have some obscure label at all, it
# would create a convenient loophole for all variables since we don't
# know anything about it. Therefore we only allow labels that have
# been seen at least once.
print "{X:SYN} : '_Label-Existenz' : auto : 0.0 :\n";
$no = 0;
for (sort keys %labels) {
  print " |\n" if $no++;
  print "  X.label = '$_'";
}
print ";\n\n";

# Direction
for (sort keys %labels) {
  my ($left, $nil, $right, $total);
  $left = $stat{directions}{$_}{-1} || 0;
  $nil = $stat{directions}{$_}{0} || 0;
  $right = $stat{directions}{$_}{1} || 0;
  my $max = $left;
  $max = $right if $right > $max;
  $max = $nil if $nil > $max;
  my $sl = $left / $max;
  my $sr = $right / $max;
  my $sn = $nil / $max;

  print "{X:SYN} : '_$_ steht links' : auto : $sl : X.label = '$_' -> ~ X/;\n"
    if $sl < 1.0;
  print "{X:SYN} : '_$_ steht rechts' : auto : $sr : X.label = '$_' -> ~ X\\;\n"
    if $sr < 1.0;
  print "{X:SYN} : '_$_ ist NIL' : auto : $sn : X.label = '$_' -> X!;\n"
    if $sn < 1.0;
}

# Distance
for (sort keys %labels) {
  my $sum = 0;
  my $no = 0;
  for my $d (keys %{$stat{distances}{$_}}) {
    my $v = $stat{distances}{$_}{$d};
    $no += $v;
    $sum += abs($d)*$v;
  }
  my $g = 50 * int($sum/$no);
  if ($g) {
    print "{X!SYN} : '_$_-Distanz' : auto :".
      "[ $g / [ $g + abs( distance(X\@id, X^id) ) ] ] :\n";
    print " X.label = '$_' -> abs(distance(X\@id,X^id)) = 1;\n";
  }
}

# Projectivity
sub label_coocurrence {
  my $h = shift;
  my $no = 0;
  my $result = '';
  for(sort keys %$h) {
    /(.+):(.+)/;
    $result .= " |\n" if $no++;
    $result .= "  X.label = $1 & Y.label = $2";
  }
  $result = 'false' unless $result;
  return $result;
}

print "{X/SYN\\Y\\SYN} : _Projektivitt1 : auto : 0.0 :\n";
print "  Y^from > X\@from ->\n";
print label_coocurrence $stat{proj1};
print ";\n\n";

print "{X\\SYN\\Y/SYN} : _Autoektivitt2 : auto : 0.0 :\n";
print "  Y^from < X\@from ->\n";
print label_coocurrence $stat{proj2};
print ";\n\n";

print "{X\\SYN||Y\\SYN} : _Projektivitt3 : auto : 0.0 :\n";
print "  X^from < Y^from & X\@from > Y^from & Y\@from > X\@from ->\n";
print label_coocurrence $stat{proj3};
print ";\n\n";

print "{X\\SYN||Y/SYN} : _Projektivitt4 : auto : 0.0 :\n";
print "  X^from < Y\@from & X\@from > Y\@from & Y^from > X\@from ->\n";
print label_coocurrence $stat{proj4};
print ";\n\n";

print "{X/SYN||Y\\SYN} : _Projektivitt5 : auto : 0.0 :\n";
print "  X\@from < Y^from & X^from > Y^from & Y\@from > X^from ->\n";
print label_coocurrence $stat{proj5};
print ";\n\n";

print "{X/SYN||Y/SYN} : _Projektivitt6 : auto : 0.0 :\n";
print "  X\@from < Y\@from & X^from > Y\@from & Y^from > X^from ->\n";
print label_coocurrence $stat{proj6};
print ";\n\n";

print "{X!SYN~=Y!SYN} : _Zyklus : auto : 0.0 : false;\n\n";


# Punctation
for (sort keys %labels) {
  my $pro = $stat{punc}{$_}{1} || 0;
  my $con = $stat{punc}{$_}{0} || 0;

  if($pro > 2*$con) {
    my $score = ($con+1) / ($pro+1);
    if ($score < 1.0) {
      print "{X!SYN} : '_$_ ohne Komma' : auto : $score :\n";
      print "  X.label = '$_' -> between(X\@id,X^id,\".,)(-\");\n\n";
    }
  }
  if($pro && $con > 2*$pro) {
    my $score = ($pro+1) / ($con+1);
    if ($score < 1.0) {
      print "{X!SYN} : '_$_ mit Komma' : auto : $score :\n";
      print "  X.label = '$_' -> ~between(X\@id,X^id,\".,)(-\");\n\n";
    }
  }
}

# Order
print "{X!SYN/\\Y!SYN} : '_Label-Label-Reihenfolge' : auto : \n";
print " [ lookup(llr, X.label, Y.label) ] : lookup(llr, X.label, Y.label) < 1.0 -> X\@from < Y\@from;\n\n";
print "llr =>\n";
$no = 0;
for my $p(sort keys %labels) {
  for my $q(sort keys %labels) {
    my $pre = $stat{order}{"$p:$q"}{pre} || 0;
    my $post = $stat{order}{"$p:$q"}{post} || 0;
    my $score = 1.0;
    if($pre > 2*$post) {
      $score = ($post+1) / ($pre+1);
    }
    print ",\n" if $no++;
    printf "'$p' '$q' => %4.3f", $score;
  }
}
print ";\n\n";


sub normalize {
  my $h = shift;
  my $max = 0;
  my %result;

  for my $k(keys %$h) {
    $max = $$h{$k} if $$h{$k} > $max;
  }
  for my $k(keys %$h) {
    $result{$k} = $$h{$k} / $max;
  }
  return %result;
}

# category/label pairs
print "{X:SYN} : '_Kategorie unter Label' : auto :\n";
print " [ lookup(kul, X\@cat, X.label) ] : lookup(kul, X\@cat, X.label) = 1.0;\n\n";
print "kul =>\n";
$no =0;
for my $c (sort keys %cats) {
  my %h = normalize $stat{botcat}{$c};
  for my $l (sort keys %labels) {
    my $score = $h{$l} || 0.0;
    print ",\n" if $no++;
    printf "'$c' '$l' => %4.3f", $score;
  }
}
print ";\n\n";

print "{X!SYN} : '_Kategorie ber Label' : auto :\n";
print " [ lookup(kuel, X^cat, X.label) ] : lookup(kuel, X^cat, X.label) = 1.0;\n\n";
print "kuel =>\n";
$no =0;
for my $l (sort keys %labels) {
  my %h = normalize $stat{topcat}{$l};
  for my $c (sort keys %cats) {
    my $score = $h{$c} || 0.0;
    print ",\n" if $no++;
    printf "'$c' '$l' => %4.3f", $score;
  }
}
print ";\n\n";


# Label co-occurrence
print "{X!SYN/\\Y!SYN} : '_Label neben Label' : auto : \n";
print " [ lookup(lnl,X.label,Y.label) ] : lookup(lnl, X.label, Y.label) = 1.0;\n\n";
print "lnl =>\n";
$no = 0;
for my $p(sort keys %labels) {
  for my $q(sort keys %labels) {
    print ",\n" if $no++;
    if(!exists $stat{cooccur}{'/\\'}{"$p:$q"}) {
      print "'$p' '$q' => 0.0";
    } else {
      print "'$p' '$q' => 1.0";
    }
  }
}
print ";\n\n";

print "{X!SYN/Y!SYN} : '_Label ber Label' : auto : \n";
print " [ lookup(lul,X.label,Y.label) ] : lookup(lul, X.label, Y.label) = 1.0;\n\n";
print "lul =>\n";
$no = 0;
for my $p(sort keys %labels) {
  for my $q(sort keys %labels) {
    print ",\n" if $no++;
    if(!exists $stat{cooccur}{'/'}{"$p:$q"}) {
      print "'$p' '$q' => 0.0";
    } else {
      print "'$p' '$q' => 1.0";
    }
  }
}
print ";\n\n";


# Tagging
print "{X:SYN} : _tagger : auto : [ pts(X\@id) ] : pts(X\@id) = 1.0;\n\n";


# Existence
for my $c(keys %cats) {
  my $count = $cats{$c};
  for my $l(keys %labels) {
    my $has = $stat{has}{$c}{$l};
    $has = 0 unless defined $has;
    my $score = 1 - ($has / $count);
    if($score < 0.5) {
      print "{X:SYN} : '_$c ohne $l' : auto : $score :\n";
      print "  X\@cat = '$c' -> has(X\@id, '$l');\n\n";
    }
  }
}
