# Copyright (C) 1997-2004 The CDG Team <cdg@nats.informatik.uni-hamburg.de>
#  
# This file is free software; as a special exception the author gives
# unlimited permission to copy and/or distribute it, with or without 
# modifications, as long as this notice is preserved.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
# implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

## ----------------------------------------------------------------------------
## AllNetworks - manage all constraint networks.
##
##
## \author Michael Daum (see also AUTHORS and THANKS for more)
## $Id: allnetworks.tcl,v 1.59 2004/10/11 13:50:05 micha Exp $
# ----------------------------------------------------------------------------
class AllNetworks {
   inherit DataBrowser

   # public methods
   public method selectIdsOfWg {wgId}; ## \type TclString
   public method getAllNetIds {}

   ## substitute for getnet
   public method getCData {id}; ## \type TclString

   public method refreshrow {row net}; ## \type TclNumber, ConstraintNet

   # protected methods
   protected method init_data {}

   constructor {args} {}; ## \type TclList

   # private methods
   private method detailsbutton_action {}
   private method deletebutton_action {}
   private method netsearchbutton_action {}
   private method frobbutton_action {}
   private method glsbutton_action {}

   # private variables
   private variable network; ## \type TclArray
};

## ----------------------------------------------------------------------------
## constructor
## ----------------------------------------------------------------------------
body AllNetworks::constructor {args} {
    set _idColumnIndex 0
    set table $itk_component(table)
    $table configure -cols 10


   itk_component add detailsbutton {
      button $itk_component(childsite).details\
		-anchor w \
		-text "Details" \
		-command [code $this detailsbutton_action]
   } { }

   itk_component add deletebutton {
      button $itk_component(childsite).delete\
		-anchor w \
		-text "Delete" \
		-command [code $this deletebutton_action]
   } { }

   itk_component add netsearchbutton {
      button $itk_component(childsite).netsearchbutton\
		-anchor w \
		-text "Search" \
		-command [code $this netsearchbutton_action]
   } { }

   itk_component add frobbutton {
      button $itk_component(childsite).frob\
		-anchor w \
		-text "Frobbing" \
		-command [code $this frobbutton_action]
   } { }
   
   itk_component add glsbutton {
      button $itk_component(childsite).gls\
		-anchor w \
		-text "Gls" \
		-command [code $this glsbutton_action]
   } { }
   

   itk_component add netsearchdialog {
      NetsearchDialog $itk_interior.netsearchdialog \
		-master . \
		-title "Xcdg: Netsearch" \
		-modality application
   } { }

   # packing
   pack \
      $itk_component(deletebutton) \
      $itk_component(netsearchbutton) \
      $itk_component(frobbutton) \
      $itk_component(glsbutton) \
      -side left -padx 3 
  # (detailsbutton remains unpacked)

   # get the initial data
   init_data
   
   # register the helpmessages
   .cdgmain help sethelpstr \
      $itk_component(table) "Click to select"

   eval itk_initialize $args
}


## ----------------------------------------------------------------------------
## get the data from the cdg-tool.
## ----------------------------------------------------------------------------
body AllNetworks::init_data {} {

  set table $itk_component(table)
  $table configure -state normal
  $table clear all
  $table delete rows 1 [$table cget -rows]
  $table tag config default
  $table tag config colored -bg gray80
  $table width 0 10 1 20

  foreach id [array names network] {
     unset network($id)
  }
  set nets  [cdgNets_get]
  _setCount [hashSize $nets]
  set hi [hashIteratorNew $nets]
  set net [hashIteratorNextValue $hi]
  while { $net != "NULL" } {
    set net_id [ConstraintNetStruct_id_get $net]
    set network($net_id) $net
    set net [hashIteratorNextValue $hi]
  }
  hashIteratorDelete $hi

  $table setCell row 0,0 { "" "Wordgraph" "Nodes" "Edges" \
       	  "Values" "Parses" "Score" "Unaries" "Binaries" \
       	  "Statistics" }

  set r 1
  foreach net_id [lsort -command ::smartCompare [array names network]] {
     set net $network($net_id)
     refreshrow $r $net
     incr r
  }

  setSelection
  $table configure -state disabled
}


## ----------------------------------------------------------------------------
## fill a row with the values from a specific net
## ----------------------------------------------------------------------------
body AllNetworks::refreshrow {row net} {
  set table $itk_component(table)

  set net_id [ConstraintNetStruct_id_get $net]
  set nodes [ConstraintNetStruct_nodes_get $net]
  set edges [ConstraintNetStruct_edges_get $net]
  set values [ConstraintNetStruct_values_get $net]
  set lattice [cnGetLattice $net]
  set lattice_id [LatticeStruct_id_get $lattice]
  set parses [ConstraintNetStruct_parses_get $net]
  set unaries [ConstraintNetStruct_evalUnary_get $net]
  set binaries [ConstraintNetStruct_evalBinary_get $net]
  set staties [ConstraintNetStruct_statUnary_get $net]
  set nonodes 0
  set noedges 0
  set novalues 0
  set noparses 0
  set agendasize 0
  set score 0

  if {$nodes != "NULL" } { set nonodes [vectorSize $nodes] }
  if {$edges != "NULL" } { set noedges [vectorSize $edges]}
  if {$values != "NULL" } { set novalues [vectorSize $values]}
  if {$parses != "NULL"} {
    set noparses [listSize $parses]
    set parse [listElement $parses]
    set score [format "%4.3e" [ParseStruct_score_get $parse]]
  }

  $itk_component(table) setCell row $row,0 [list \
       $net_id $lattice_id $nonodes $noedges \
       $novalues $noparses $score $unaries \
       $binaries $staties ]

}



## ----------------------------------------------------------------------------
## action taking place when the searchforparsebutton is pressed
## ----------------------------------------------------------------------------
body AllNetworks::netsearchbutton_action {} {
   if {"$_selection" != ""} {
      $itk_component(netsearchdialog) activate $_selection
   }
}

## ----------------------------------------------------------------------------
## action taking place when the frobbutton is pressed
## ----------------------------------------------------------------------------
body AllNetworks::frobbutton_action {} {

 foreach id $_selection {
   ::cmd::Frobbing $id method=combined pressure=0.0 batch=yes
 }

}

## ----------------------------------------------------------------------------
## action taking place when the glsbutton is pressed.
## ----------------------------------------------------------------------------
body AllNetworks::glsbutton_action {} {

  if {$_selection == ""} {
    return
  }

  foreach id $_selection {
    ::cmd::Gls $id 
  }
}

## ----------------------------------------------------------------------------
## action taking place when the deletebutton is pressed.
## ----------------------------------------------------------------------------
body AllNetworks::deletebutton_action {} {

   foreach id $_selection {
      commandEval "netdelete $id"
       puts "Delete $id"
   }
   init_data
}

## ----------------------------------------------------------------------------
## action taking place when the detailsbutton is pressed.
## ----------------------------------------------------------------------------
body AllNetworks::detailsbutton_action {} {

   .cdgmain not_yet
}


## ----------------------------------------------------------------------------
## get pointer to a net_id, cache it in array network.
## ----------------------------------------------------------------------------
body AllNetworks::getCData {id} {

   if {![info exists network($id)]} {
     set net [cnFindNet $id]
     if {$net == "NULL"} {
       return "NULL"
     }
     set network($id) $net
     #_setCount [hashSize [cdgNets_get]]
   }
   return $network($id)
}


## ----------------------------------------------------------------------------
## select nets based on a specified wordgraph.
## previous selections are cleared
## ----------------------------------------------------------------------------
body AllNetworks::selectIdsOfWg {wgId} {

   set table $itk_component(table)

   set rmax [$table cget -rows]
   $table selection clear 0,0 end
   
   set newselection ""
   for {set row 0} {$row < $rmax} {incr row} {
      if {[$table getCell $row,1] == $wgId} {
	 set id [$table getCell $row,0 ]
	 lappend newselection $id
	 $table selection set $row,0
      }
   }
   
   set _selection $newselection
}


## ----------------------------------------------------------------------------
## Return list of all nets' names
## ----------------------------------------------------------------------------
body AllNetworks::getAllNetIds {} {
  init_data
  return [array names network]
}
