# Copyright (C) 1997-2004 The CDG Team <cdg@nats.informatik.uni-hamburg.de>
#  
# This file is free software; as a special exception the author gives
# unlimited permission to copy and/or distribute it, with or without 
# modifications, as long as this notice is preserved.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
# implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

## ----------------------------------------------------------------------------
## CdgHelp - a status line.
## This class allows to display a help message in a status line whenever
## the mouse pointer enters a widget.
##
## \author Michael Daum (see also AUTHORS and THANKS for more)
## $Id: help.tcl,v 1.7 2004/02/25 14:40:42 micha Exp $
## ----------------------------------------------------------------------------
class CdgHelp {
   inherit itk::Widget

   # public methods
   public method sethelpstr {w text args}; ## \type TclCommand, TclString, TclList
   public method gethelpstr {w}; ## \type TclCommand
   public method unsethelpstr {w}; ## \type TclCommand
   public method showstr {text}; ## \type TclString
   public method show {w}; ## \type TclCommand
   public method clear {}

   constructor {args} {}; ## \type TclList
   
   # private variables
   ## a hash mapping widgets to the help message that should be displayed.
   private variable _messageOfWidget; ## \type TclArray
};

## ----------------------------------------------------------------------------
## constructor
## ----------------------------------------------------------------------------
body CdgHelp::constructor {args} {

   itk_component add label {
      label $itk_interior.label -text "" \
		-anchor w \
		-borderwidth 0
   } { 
      
      keep -font -background -relief -cursor
   }

   # packing
   pack $itk_component(label) -fill both -expand 1
   
   eval itk_initialize $args
}

## ----------------------------------------------------------------------------
## set the displayed text when the mouse enters the widget.
## install two events &lt;Enter&gt; and &lt;Leave&gt; for the widget
## ----------------------------------------------------------------------------
body CdgHelp::sethelpstr {w text args} {

   if {![info exists _messageOfWidget($w)]} {
      set no [scan $w "%s %d" base index] 
      set cl [winfo class $base]
      if {$no == 2 && $cl == "Menu"} {
	 bind $base <Motion> [code $this show %W]
	 bind $base <Leave> [code $this clear]
      } else {
	 bind $w <Enter> [code $this show %W]
	 bind $w <Leave> [code $this clear]
      }
   }
   set _messageOfWidget($w) $text

   if {$args != ""} {
      eval sethelpstr $args
   }
}

## ----------------------------------------------------------------------------
## unset the array-entry, remove the &lt;Enter&gt;- and &lt;Leave&gt;-bindings
## ----------------------------------------------------------------------------
body CdgHelp::unsethelpstr {w} {

   if {![info exists _messageOfWidget($w)]} {
      unset _messageOfWidget($w)
      bind $w <Enter> {}
      bind $w <Leave> {}
   }
}

## ----------------------------------------------------------------------------
## returns the helpstring of a given widget
## ----------------------------------------------------------------------------
body CdgHelp::gethelpstr {w} {
   return $_messageOfWidget($w)
}

## ----------------------------------------------------------------------------
## displays the a noregistered message
## ----------------------------------------------------------------------------
body CdgHelp::showstr {text} {
   $itk_component(label) configure -text $text
}

## ----------------------------------------------------------------------------
## displays the associated helpmessage
## ----------------------------------------------------------------------------
body CdgHelp::show {w} {

   if {[winfo class $w] == "Menu"} {
      set index [$w index active]
      append w " $index"
   }
   set text ""
   catch {set text $_messageOfWidget($w)}
   $itk_component(label) configure -text $text
}

## ----------------------------------------------------------------------------
## clear the display
## ----------------------------------------------------------------------------
body CdgHelp::clear {} {
   $itk_component(label) configure -text ""
}

