# Copyright (C) 1997-2004 The CDG Team <cdg@nats.informatik.uni-hamburg.de>
#  
# This file is free software; as a special exception the author gives
# unlimited permission to copy and/or distribute it, with or without 
# modifications, as long as this notice is preserved.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
# implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

## ----------------------------------------------------------------------------
## CdgPrefs - a preference dialog
##
## \author Michael Daum (see also AUTHORS and THANKS for more)
## $Id: prefs.tcl,v 1.12 2004/02/25 14:40:42 micha Exp $
## ----------------------------------------------------------------------------
class CdgPrefs {
   inherit iwidgets::Dialog

   # public methods
   public method activate {}

   constructor {args} {}; ## \type TclList
   
   # private methods
   private method _ok_action {}
   private method _getData {}
   private method _setData {}

   # private variables

   ## prefenrence-data mirrored from CdgMain
   private variable _preferences	;## \type TclArray
};

## ----------------------------------------------------------------------------
## constructor
## ----------------------------------------------------------------------------
body CdgPrefs::constructor {args} {

   hide Apply
   hide Help
   buttonconfigure OK -command [code $this _ok_action]

   set child [childsite]

   itk_component add grammarpath {
      iwidgets::entryfield $child.grammarpath \
		-labeltext "Grammar-Path: "\
		-labelpos w \
		-textvariable [scope _preferences(grammarpath)]
   } {}

   itk_component add editor {
      iwidgets::entryfield $child.editor \
		-labeltext "Editor: "\
		-labelpos w \
		-textvariable [scope _preferences(editor)]
   } {}

   iwidgets::Labeledwidget::alignlabels \
		$itk_component(grammarpath) \
		$itk_component(editor)

   pack $itk_component(grammarpath) \
	$itk_component(editor) \
	-side top -expand 1 -fill both -padx 5 -pady 5


   eval itk_initialize $args
}

## ----------------------------------------------------------------------------
## involved by buttonpress on ok-button
## ----------------------------------------------------------------------------
body CdgPrefs::_ok_action {} {
  _setData
  deactivate
}


## ----------------------------------------------------------------------------
## init data, start dialog
## ----------------------------------------------------------------------------
body CdgPrefs::activate {} {
   center .
   _getData
   Shell::activate
}

## ----------------------------------------------------------------------------
## refresh _preferences from main
## ----------------------------------------------------------------------------
body CdgPrefs::_getData {} {
   set _preferences(grammarpath) [.cdgmain cget -grammarpath]
   set _preferences(editor) [.cdgmain cget -editor]
}

## ----------------------------------------------------------------------------
## set configurations
## ----------------------------------------------------------------------------
body CdgPrefs::_setData {} {
  .cdgmain configure -grammarpath $_preferences(grammarpath)
  .cdgmain configure -editor $_preferences(editor)
}
