# Copyright (C) 1997-2004 The CDG Team <cdg@nats.informatik.uni-hamburg.de>
#  
# This file is free software; as a special exception the author gives
# unlimited permission to copy and/or distribute it, with or without 
# modifications, as long as this notice is preserved.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
# implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

## ----------------------------------------------------------------------------
## StartUp - startup-screen
##
## \author Dietmar Fnning, Michael Daum
## $Id: startup.tcl,v 1.7 2004/10/11 15:23:31 micha Exp $
## ----------------------------------------------------------------------------
class StartUp {
   inherit ::itk::Toplevel

   itk_option define -height height Height 798
   itk_option define -width width Width 599

   # public
   public method step {n}; ## \type TclNumber

   destructor {}
   constructor {args} {}; ## \type TclList

   # private
   private method _drawScreen {}

   private variable _picture; ## \type TclCommand
   private variable _red 4095
   private variable _green 0
   private variable _progress 0
   private variable _oldProgress 0
   private variable _ratio 0
   private variable _percent "0%"
   private variable _stepTimer 0
   private variable _totalTimer 0
};

##-----------------------------------------------------------------------------
## destructor
##-----------------------------------------------------------------------------
body StartUp::destructor {} {

   set now [clock clicks]
   set duration [expr ($now - $_totalTimer) / 1000.0]
#   puts "PROFILE: total startup $duration ms"

    image delete $_picture
}

## ----------------------------------------------------------------------------
## constructor
## ----------------------------------------------------------------------------
body StartUp::constructor {args} {

  global env

  set _picture [image create photo $this.picture -file $env(XCDG_LIB)/xcdglogo.gif]

    itk_component add canvas {
      canvas $itk_interior.canvas -background white
    } {}

    pack $itk_component(canvas) \
		-fill both \
		-expand 1
     
    eval itk_initialize $args

    set wd [winfo screenwidth $itk_component(hull)]
    set ht [winfo screenheight $itk_component(hull)]
    set rwd $itk_option(-height)
    set rht $itk_option(-width)
    set x [expr ($wd - $rwd) / 2 ]
    set y [expr ($ht - $rht)  / 2 ]
    wm geometry $itk_component(hull) "+$x+$y"

    _drawScreen
    update idletask

    set _stepTimer [clock clicks]
    set _totalTimer $_stepTimer
}

##-----------------------------------------------------------------------------
## option -height.
## set the overall height of the application
##-----------------------------------------------------------------------------
configbody StartUp::height {
   wm geometry $itk_component(hull) \
      $itk_option(-height)x$itk_option(-width)
}

##-----------------------------------------------------------------------------
## option -width.
## set the overall width of the application
##-----------------------------------------------------------------------------
configbody StartUp::width {
   wm geometry $itk_component(hull) \
      $itk_option(-height)x$itk_option(-width)
}

##-----------------------------------------------------------------------------
## well, draw the opening screen
##-----------------------------------------------------------------------------
body StartUp::_drawScreen {} {
  
   set cv $itk_component(canvas)
   set height $itk_option(-height)
   set width $itk_option(-width)
   set midPos [expr $width / 2.0 + 85]

   $cv create image -11 -11 -image $_picture -anchor nw

   set x1 [expr $midPos - 260]
   set y1 [expr $height / 2.0 -150 - 7]
   set x2 [expr $midPos + 280]
   set y2 [expr $height / 2.0 -150 + 7]
   set _ratio [expr ($x2 - $x1) / 100.0]

   $cv create rect $x1 $y1 $x2 $y2  -fill gray60 \
                    -tag feedyBackground
   $cv create rect $x1 $y1 $x1 $y2 \
                    -fill red \
                    -tag feedyForeground	

   set midPos [expr $midPos + 7]
   $cv create text $midPos 320  -text $_percent \
                    -font *times-medium-r-*-*-18* -tag Percent
}  

##-----------------------------------------------------------------------------
## update and display progress
##-----------------------------------------------------------------------------
body StartUp::step { n } {
   set cv $itk_component(canvas)
   set _progress $n
 
     set bbox [$cv coords feedyForeground] 
     scan $bbox "%f %f %f %f" x1 y1 x2 y2

     set i $_oldProgress
     while { $i < $_progress } {
       incr i
       set _red [expr $_red - 18]
       set _green [expr $_green + 40]
       set color [format "#%03x%03x%03x" $_red $_green 1100 ]
       $cv coords feedyForeground $x1 $y1 [expr $x1 + $i * $_ratio] $y2
       $cv itemconfigure feedyForeground -fill $color
       $cv itemconfigure Percent -text [format "%3d%%" $i]
       update idletask
     }
     set _oldProgress $_progress


   set now [clock clicks]
   set duration [expr ($now - $_stepTimer) / 1000.0]


#   puts "PROFILE: step $n after $duration ms ([expr ($duration / 2729.726) * 100] %)"
}


