# Copyright (C) 1997-2004 The CDG Team <cdg@nats.informatik.uni-hamburg.de>
#  
# This file is free software; as a special exception the author gives
# unlimited permission to copy and/or distribute it, with or without 
# modifications, as long as this notice is preserved.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
# implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

## ----------------------------------------------------------------------------
## YadaConfigDocument - abstract base class for all configure documents.
## \ingroup YadaConfiguration
##
## \author Michael Daum
##
## $Id: YadaConfigDocument.tcl,v 1.10 2004/02/26 14:23:36 ddreyer Exp $
## ----------------------------------------------------------------------------
class YadaConfigDocument {
  inherit itk::Widget YadaPlainDocument

  # variables ----------------------------------------------------------------
  protected variable _oldItem "<none>"
  protected variable _allAspects ""

  private variable _currentItem ""
  private variable _allItems; ## \type TclArray
  private variable _currentItemNo 0
  private variable _counter "0/0"

  # methods ------------------------------------------------------------------
  public method init {}
  public method activationHandle {}
  public method getItem {name}; ## \type TclString
  public method getItemNo {name}; ## \type TclString
  public method getAllItems {args}; ## \type TclList
  public method getAllItemNames {args}; ## \type TclList
  public method setItem {item}; ## \type YadaConfigItem
  public method unsetItem {item}; ## \type YadaConfigItem
  public method setCurrentItem {item}; ## \type YadaConfigItem
  public method getCurrentItem {}
  public method displayTitle {}

  constructor {args} {}; ## \type TclList
  destructor {}
  

  private method _undo {}
  private method _selectCommand {}
  private method _next {}
  private method _prev {}
  private method _setCounter {}
  

  protected method _askChange {}
  protected method _mark {aspect {newData "undef"} {oldData "undef"}}; ## \type TclString, TclString, TclString
  protected method _unmark {{aspect ""}}; ## \type TclString
  protected method _getMarked {}
  protected method _commit {} 

  protected method _select {{itemName ""}} ;## \purevirtual \type TclString
  protected method _new {} ; ## \purevirtual
  protected method _delete {} ;## \purevirtual
  protected method _defaults {{fileName ""}} ;## \purevirtual \type TclString
  protected method _validateCommand {aspect newValue {oldValue ""}}; ## \type TclString, TclString, TclString

  protected method isValidPersistanceFile {fileName}; ## \purevirtual \type TclString

};

## ----------------------------------------------------------------------------
## constructor
## ----------------------------------------------------------------------------
body YadaConfigDocument::constructor {args} {
  #
  # create custom file menu
  #
  set fileButton [.main component fileButton]
  for {
    set i 1
    set fileMenu $fileButton.menu$i
  } {[winfo exists $fileMenu]} {incr i} {
    set fileMenu $fileButton.menu$i
  }
  itk_component add fileMenu {
      menu $fileMenu
  } {
      keep -background -cursor
      rename -borderwidth -menuborderwidth menuBorderwidth Borderwidth
  }

  ## create submenu entry "New"
  itk_component add newMenu {
     menu $itk_component(fileMenu).newMenu 
  } {
     keep -background -cursor
     rename -borderwidth -menuborderwidth menuBorderwidth Borderwidth
  }

  ## add file menu entries
  $itk_component(fileMenu) add command \
       	-label "Load" \
       	-underline 0 \
       	-command [code $this load] 
  $itk_component(fileMenu) add command \
       	-label "Save" \
       	-underline 0 \
       	-command [code $this save] 
  $itk_component(fileMenu) add cascade \
	-label "New" \
	-underline 0 \
	-menu $itk_component(newMenu)
  $itk_component(fileMenu) add separator
  $itk_component(fileMenu) add command \
	-label "Quit" \
	-underline 0 \
	-command {.main quit}


  $itk_component(newMenu) add command \
	-label "Runner" \
	-underline 0 \
	-command {YadaDocument::newDocument YadaRunner Runner}
  $itk_component(newMenu) add command \
	-label "Difference" \
	-underline 0 \
	-command {YadaDocument::newDocument YadaDifference Difference}
  $itk_component(newMenu) add command \
	-label "Gls Statistics" \
	-underline 0 \
	-command {YadaDocument::newDocument YadaGlsStats GlsStatistics}
  $itk_component(newMenu) add command \
	-label "Ranking" \
	-underline 0 \
	-command {YadaDocument::newDocument YadaRanking Ranking}
 

  # edit menu
  set editButton [.main component editButton]
  for {
    set i 1
    set editMenu [subst $editButton.menu$i]
  } {[winfo exists $editMenu]} {incr i} {
    set editMenu [subst $editButton.menu$i]
  }
  
  itk_component add editMenu {
     menu $editMenu
  } {
     keep -background -cursor
     rename -borderwidth -menuborderwidth menuBorderwidth Borderwidth
  }

  $itk_component(editMenu) add command \
       	-label "Undo" \
       	-underline 0 \
       	-command [code $this _undo]
  $itk_component(editMenu) add command \
       	-label "Commit" \
       	-underline 0 \
       	-command [code $this _commit]
  $itk_component(editMenu) add command \
       	-label "New" \
       	-underline 0 \
       	-command [code $this _new]
  $itk_component(editMenu) add command \
       	-label "Delete" \
       	-underline 0 \
       	-command [code $this _delete]
  $itk_component(editMenu) add separator
  $itk_component(editMenu) add command \
       	-label "Find" \
       	-underline 0 \
       	-command [code $this _find] \
	-state disabled

  # buttons
  itk_component add buttonFrame {
    frame $itk_interior.buttonFrame
  }
  itk_component add topButtonFrame {
    frame $itk_interior.buttonFrame.top
  }
  itk_component add botButtonFrame {
    frame $itk_interior.buttonFrame.bot
  }
  itk_component add commitButton {
    button $itk_component(topButtonFrame).commitButton \
	-text "Commit" \
	-command [code $this _commit]
  }
  itk_component add newButton {
    button $itk_component(topButtonFrame).newButton \
	-text "New" \
	-command [code $this _new]
  }
  itk_component add deleteButton {
    button $itk_component(topButtonFrame).deleteButton \
	-text "Delete" \
	-command [code $this _delete]
  }
  itk_component add undoButton {
    button $itk_component(topButtonFrame).undoButton \
      -text "Undo" \
      -command [code $this _undo]
  }
  itk_component add naviFrame {
    frame $itk_component(botButtonFrame).naviFrame
  }
  itk_component add nextButton {
    button $itk_component(naviFrame).nextButton \
      -text ">" \
      -command [code $this _next]
  }
  itk_component add prevButton {
    button $itk_component(naviFrame).prevButton \
      -text "<" \
      -command [code $this _prev]
  }

  # counter
  itk_component add counter {
    label $itk_interior.counter \
      -textvariable [scope _counter] \
      -anchor se  \
      -font {-adobe-helvetica-medium-r-normal--10-*-*-*-*-*-*-*}
  } { }

  # childsite
  itk_component add childsite {
    frame $itk_interior.childsite 
  } 

  # name
  itk_component add nameLabel {
    label $itk_component(childsite).nameLabel \
      -text "Name:" \
      -anchor nw
  } {}

  itk_component add nameMark {
    frame $itk_component(childsite).nameMark \
      -borderwidth 2
  } {}

  itk_component add name {
    iwidgets::combobox $itk_component(nameMark).name \
      -completion false \
      -borderwidth 2 \
      -unique true \
      -selectioncommand [code $this _selectCommand]
  }  {}
  set entry [$itk_component(name) component entry]
  $entry configure \
    -validate all \
    -validatecommand [code $this _validateCommand "name" %P]


  #
  # packing + gridding
  #
  grid columnconfigure $itk_interior 0 -weight 1
  grid rowconfigure $itk_interior 0 -weight 1
  grid columnconfigure $itk_component(childsite) 1 -weight 1
  grid $itk_component(childsite) -sticky  news -padx 10 -pady 10 -row 0 -column 0 -rowspan 2
  grid $itk_component(buttonFrame) -sticky ns -padx 10 -row 0 -column 2 -pady 10
  grid $itk_component(counter) -sticky se -padx 0 -row 1 -column 2 -pady 5 -padx 5

  grid $itk_component(nameLabel) -sticky ew -row 0 -column 0 
  grid $itk_component(nameMark) -sticky ew -row 0 -column 1 -pady 10 -columnspan 2
  pack $itk_component(name) -fill both -expand 1

  pack $itk_component(topButtonFrame) -side top -fill both -expand 1
  pack $itk_component(botButtonFrame) -side top -fill both -expand 1
  pack $itk_component(naviFrame) -side bottom -fill x
  pack $itk_component(commitButton) -side top -fill x -pady 5
  pack $itk_component(undoButton) -side top -fill x -pady 5
  pack $itk_component(newButton) -side top -fill x -pady 5
  pack $itk_component(deleteButton) -side top -fill x -pady 5
  pack $itk_component(prevButton) -side left -fill x -pady 0 
  pack $itk_component(nextButton) -side right -fill x -pady 0


  # bindings
  set entry [$itk_component(name) component entry]
  bind $entry <Up> +[code $this _selectCommand]
  bind $entry <Down> +[code $this _selectCommand]

  eval itk_initialize $args
}

## ----------------------------------------------------------------------------
## destructor
## ----------------------------------------------------------------------------
body YadaConfigDocument::destructor {} {
  foreach item [array names _allItems] {
    itcl::delete $_allItems($item)
  }
}

## ----------------------------------------------------------------------------
## init
## ----------------------------------------------------------------------------
body YadaConfigDocument::init {} {
  if {$_isInitialized} {
    return
  }
  chain

  if {$name != ""} {

    # load resource file
    set rcFileName [file join $::env(YADA_ETC) $name.tcl]
    if {[file exists $rcFileName]} {
      .main printStatus "... loading $name file `$rcFileName'."
      _defaults $rcFileName
    } else {
      .main printStatus "... configure file `$rcFileName' not found ... using defaults."
      _defaults
    }
  }

  # initialize the name combobox 
  eval $itk_component(name) insert list end [getAllItemNames]
}

## ----------------------------------------------------------------------------
## _selectCommand
## ----------------------------------------------------------------------------
body YadaConfigDocument::_selectCommand {} {
  _unmark
  after idle [code $this _select]
}

## ----------------------------------------------------------------------------
## _askChange
## ----------------------------------------------------------------------------
body YadaConfigDocument::_askChange {} {
  set result 1

  set markedAspects [_getMarked]
  if {$markedAspects != ""} {
    set msg [iwidgets::messagedialog .msg \
      -text "Are you sure that you want to discard your changes?" \
      -title "Yada - Caution"  \
      -image [YadaImages::get question] \
      -modality application \
      -master $itk_interior]
    $msg buttonconfigure OK -text "Yes"
    $msg buttonconfigure Cancel -text "No"
    $msg center .
    set result [$msg activate]
    destroy $msg

#    if {!$result} {
#      set box $itk_component(name)
#      $box delete entry 0 end
#      $box insert entry 0 $_oldItem
#    }
  }

  return $result
}

## ----------------------------------------------------------------------------
## activationHandle
## ----------------------------------------------------------------------------
body YadaConfigDocument::activationHandle {} {
  chain

    ## activate custom file menu
    set fileButton [.main component fileButton]
    $fileButton configure -menu $itk_component(fileMenu)

    [getCurrentItem] displayTitle
   
  # set the menus
  set editButton [.main component editButton]
  $editButton configure -menu $itk_component(editMenu)
}

## ----------------------------------------------------------------------------
## _unmark
## ----------------------------------------------------------------------------
body YadaConfigDocument::_unmark {{aspect ""}} {
  if {$aspect == ""} {
    foreach aspect $_allAspects {
      $itk_component(${aspect}Mark) configure -background gray90
    }
  } else {
    $itk_component(${aspect}Mark) configure -background gray90
  }
}

## ----------------------------------------------------------------------------
## _mark
## ----------------------------------------------------------------------------
body YadaConfigDocument::_mark {aspect {newData "undef"} {oldData "undef"}} {
  set itemName [$itk_component(name) get]
  if {$itemName == "<none>" || $itemName == ""} {
    return
  }

  if {$newData == "undef"} {
    set newData [$itk_component($aspect) get]
  }
  if {$oldData == "undef"} {
    set oldData [string trim [$_currentItem cget -$aspect]]
  }

  set mark $itk_component(${aspect}Mark)

  if {$newData != $oldData} {
    $mark configure -background red4
  } else {
    $mark configure -background gray90
  }
}

## ----------------------------------------------------------------------------
## _getMarked
## ----------------------------------------------------------------------------
body YadaConfigDocument::_getMarked {} {
  set result ""
  foreach aspect $_allAspects {
     if {[$itk_component(${aspect}Mark) cget -background] == "red4"} {
      lappend result $aspect
     }
  }

  return $result
}

## ----------------------------------------------------------------------------
## _undo
## ----------------------------------------------------------------------------
body YadaConfigDocument::_undo {} {
  if {$_currentItem != ""} {
    _select [$_currentItem cget -name]
  }
}

## ----------------------------------------------------------------------------
## _validateCommand
## ----------------------------------------------------------------------------
body YadaConfigDocument::_validateCommand {aspect newValue {oldValue ""}} {
  if {$_currentItem == ""} {
    return
  }

  if {$oldValue == ""} {
    set oldValue [string trim [$_currentItem cget -$aspect]]
  }

  set mark $itk_component(${aspect}Mark)
  if {$newValue != $oldValue} {
    $mark configure -background red4
  } else {
    $mark configure -background gray90
  }

  return 1
}

## ----------------------------------------------------------------------------
## _next
## ----------------------------------------------------------------------------
body YadaConfigDocument::_next {} {
  set itemNames [getAllItemNames]
  set index $_currentItemNo
  incr index
  set itemName [lindex $itemNames $index]
  if {$itemName != ""} {
    _select $itemName
  }
}

## ----------------------------------------------------------------------------
## _prev
## ----------------------------------------------------------------------------
body YadaConfigDocument::_prev {} {
  set itemNames [getAllItemNames]
  set index $_currentItemNo
  incr index -1
  set itemName [lindex $itemNames $index]
  if {$itemName != ""} {
    _select $itemName
  }
}

## ----------------------------------------------------------------------------
## getCurrentItem
## \returns the currently displayed YadaConfigItem
## ----------------------------------------------------------------------------
body YadaConfigDocument::getCurrentItem {} {
  return $_currentItem
}

## ----------------------------------------------------------------------------
## update the counter label.
## This method sets the _counter to reflect the number of items in this document.
## ----------------------------------------------------------------------------
body YadaConfigDocument::_setCounter {} {
  set noItems [llength [array names _allItems]]
  if {$noItems == 0} {
    set _counter "0/0"
  } else {
    set _counter "[expr $_currentItemNo +1]/$noItems"
  }
}

## ----------------------------------------------------------------------------
## setCurrentItem
## ----------------------------------------------------------------------------
body YadaConfigDocument::setCurrentItem {item} {
  if {$item == ""} {
    set firstName [lindex [getAllItemNames] 0]
    set _currentItem [getItem $firstName]
  } else {
    setItem $item
  }

  set _currentItem $item

  if {$_currentItem == ""} {
    set _currentItemNo "0"
  } else {
    set _currentItemNo [getItemNo [$item cget -name]]
  }
  _setCounter
}

## ----------------------------------------------------------------------------
## unsetItem
## ----------------------------------------------------------------------------
body YadaConfigDocument::unsetItem {item} {
  set itemName [$item cget -name]

  if {![info exists _allItems($itemName)]} {
    return
  }

  if {$_currentItem == $item} {
    setCurrentItem ""
  }

  unset _allItems($itemName)
  _setCounter
}

## ----------------------------------------------------------------------------
## setItem
## ----------------------------------------------------------------------------
body YadaConfigDocument::setItem {item} {
  if {$item == ""} {
    return
  }

  set itemName [$item cget -name]
  set _allItems($itemName) $item
  _setCounter
}

## ----------------------------------------------------------------------------
## getItemNo
## ----------------------------------------------------------------------------
body YadaConfigDocument::getItemNo {itemName} {
  set allNames [getAllItemNames]
  return [lsearch $allNames $itemName]
}

## ----------------------------------------------------------------------------
## getItem
## ----------------------------------------------------------------------------
body YadaConfigDocument::getItem {itemName} {
  if {[info exists _allItems($itemName)]} {
    return $_allItems($itemName)
  } else {
    return ""
  }
}

## ----------------------------------------------------------------------------
## getAllItems
## \returns a list of all known YadaConfigItems in this document
## ----------------------------------------------------------------------------
body YadaConfigDocument::getAllItems {args} {
  set result ""

  foreach itemName [eval getAllItemNames $args] {
    lappend result $_allItems($itemName)
  }

  return $result
}

## ----------------------------------------------------------------------------
## getAllItemNames
## \returns a list of all known names of YadaConfigItems in this document
## ----------------------------------------------------------------------------
body YadaConfigDocument::getAllItemNames {args} {
  set result ""

  if {[llength $args] == 0} {
    return [lsort -dictionary [array names _allItems]]
  } 

  foreach itemName [lsort -dictionary [array names _allItems]] {
    foreach pattern $args {
      if {[string match $pattern $itemName]} {
	lappend result $itemName
	break
      }
    }
  }

  return $result
}

## ----------------------------------------------------------------------------
## _commit
## ----------------------------------------------------------------------------
body YadaConfigDocument::_commit {} {

  # check new item name
  set newName [$itk_component(name) get]
  set oldName [$_currentItem cget -name]

  set listBox [$itk_component(name) component list]
  set knownNames [$listBox get 0 end]
  
  if {$newName != $oldName} {
    if {[info exists _allItems($newName)]} {
      printMessage "ERROR: '$newName' exists already." error
      return 0
    }

    # delete the old item name
    set index [lsearch $knownNames $oldName]
    $listBox delete $index $index
    set knownNames [lreplace $knownNames $index $index]

    # relockate item in array
    unset _allItems($oldName)
    set _allItems($newName) $_currentItem
    $_currentItem configure -name $newName
    set _currentItemNo [getItemNo $newName]
    _setCounter
  }

  # save name unique in comboboxes
  if {[getIndexOfName $listBox $newName] < 0} {
    lappend knownNames $newName
    $listBox delete 0 end
    eval $listBox insert 0 [lsort -dictionary $knownNames]
  }

  return 1
}

## ----------------------------------------------------------------------------
## Update toolbar, menubar and titlebar
## ----------------------------------------------------------------------------
body YadaConfigDocument::displayTitle {} {
 
    set currentItem [getCurrentItem]
    set loadButton [[[.main getDocument Configure] component toolbar] component loadButton]
    set saveButton [[[.main getDocument Configure] component toolbar] component saveButton]
   

    if { $currentItem != "" } {
	$currentItem displayTitle

	if { [$currentItem hasSavedState] } {
	    $itk_component(fileMenu) entryconfigure 0 -state normal
	    $loadButton configure -state normal
	} else {
	    $itk_component(fileMenu) entryconfigure 0 -state disabled
	    $loadButton configure -state disabled
	}

	if { [$currentItem isModified] && [string compare [$currentItem cget -name] "<none>"] != 0 } {
	    $itk_component(fileMenu) entryconfigure 1 -state normal   
	    $saveButton configure -state normal
	} else {
	    $itk_component(fileMenu) entryconfigure 1 -state disabled
	    $saveButton configure -state disabled
	}
    }
}


